#!/usr/bin/env python
# -*- coding:UTF-8 -*-
'''
@Description:
	Perform metagene plot around start and stop codon in codon units or nt units
	Note:
	1) Only input sorted and indexed BAM file(s). SAM format is not supported.
	2) Acoording to one gene may has serveral isoforms , we select the longest isoform.
	3) The selected isoform whose CDS length is not multiple to 3 duing to programmed ribosome frameshifting were excluded also.
'''

from __future__ import division
from .FunctionDefinition import *
import itertools
from scipy import stats
from collections import defaultdict
import matplotlib
matplotlib.use('Agg')
import matplotlib.pyplot as plt
from matplotlib.backends.backend_pdf import PdfPages
import seaborn as sns

def CI_for_t_distribution(data,confidence=0.95):
	'''
	scripts source:
	https://www.jianshu.com/p/6cfce4cc2f7f
	'''
	mean=np.mean(data)
	std=np.std(data,ddof=1)
	size=len(data)
	alpha=1-confidence
	t_score=stats.t.isf(alpha/2,df=(size-1))
	ME=t_score*std/np.sqrt(size)
	lower_CI=mean-ME
	upper_CI=mean+ME
	return lower_CI,upper_CI

def ribosomeDensityNormPerTrans(in_bamFile,in_selectTrans,in_transLengthDict,in_startCodonCoorDict,in_stopCodonCoorDict,inCDS_lengthFilterParma,inCDS_countsFilterParma,in_regionLengthParma,in_extendRegionLengthParma,in_excludeLengthParma,in_excludeCodonCountsParma,in_readLengths,in_readOffset,Mode,Unit,Type,confidence,norm):
	pysamFile=pysam.AlignmentFile(in_bamFile,"rb")
	pysamFile_trans=pysamFile.references
	in_selectTrans=set(pysamFile_trans).intersection(in_selectTrans)
	passTransSet=set()
	startDensity=np.zeros(int(in_regionLengthParma+in_extendRegionLengthParma+1),dtype="float64")
	stopDensity =np.zeros(int(in_regionLengthParma+in_extendRegionLengthParma+1),dtype="float64")
	start_lower_CI=np.zeros(int(in_regionLengthParma+in_extendRegionLengthParma+1),dtype="float64")
	start_upper_CI=np.zeros(int(in_regionLengthParma+in_extendRegionLengthParma+1),dtype="float64")
	stop_lower_CI=np.zeros(int(in_regionLengthParma+in_extendRegionLengthParma+1),dtype="float64")
	stop_upper_CI=np.zeros(int(in_regionLengthParma+in_extendRegionLengthParma+1),dtype="float64")
	startNormedWindowsList=[]
	startWindowsExistPosList=[]
	stopNormedWindowsList=[]
	stopWindowsExistPosList=[]
	i=len(in_selectTrans)
	filter_1=0
	filter_2=0
	filter_3=0
	filter_4=0
	filter_5=0
	all_counts=0
	startNormedWindowsDict={}
	for trans in in_startCodonCoorDict.keys():
		leftCoor =int(in_startCodonCoorDict[trans])-1
		rightCoor=int(in_stopCodonCoorDict[trans])-3
		(trans_counts,read_counts_frameSum,total_reads,cds_reads)=get_trans_frame_counts(pysamFile, trans, in_readLengths, in_readOffset, in_transLengthDict[trans], leftCoor, rightCoor)
		all_counts+=total_reads ## total_reads for transcript level
	## codon unit
	if Unit=="codon":
		for trans in in_selectTrans:
			i-=1
			leftCoor =int(in_startCodonCoorDict[trans])-1 #the first base of start codon 0-base
			rightCoor=int(in_stopCodonCoorDict[trans])-3 #the first base of stop codon 0-base
			# cdsLength should obey : (1) >= minFilterLength codon unit (2) CDSlength=3n
			if ((rightCoor-leftCoor) < inCDS_lengthFilterParma*3):
				filter_1+=1
				continue
			if (rightCoor-leftCoor)%3 !=0:
				filter_2+=1
				continue
			# get transNum and trans 5Pos offset counts vector
			(trans_counts,read_counts_frameSum,total_reads,cds_reads)=get_trans_frame_counts(pysamFile, trans, in_readLengths, in_readOffset, in_transLengthDict[trans], leftCoor, rightCoor)
			if total_reads==0 or in_transLengthDict[trans]==0:
				filter_3+=1
				continue
			cds_reads_normed=10**9*(cds_reads/(all_counts*len(read_counts_frameSum)))
			# cds num
			if Mode=="RPKM":
				if cds_reads_normed < inCDS_countsFilterParma:
					filter_4+=1
					continue
			elif Mode=="counts":
				if cds_reads < inCDS_countsFilterParma:
					filter_4+=1
					continue
			else:
				raise KeyError("There is no such modes, please check it.['RPKM' or 'counts']")

			# normalize the counts for per transcript
			read_counts_frameSum_normed=10**6*(read_counts_frameSum/(all_counts))
			# read_counts_frameSum_normed=10**9*(read_counts_frameSum/(all_counts*len(read_counts_frameSum)))
			trans_counts_normed=10**6*(trans_counts/(all_counts))
			# trans_counts_normed=10**9*(trans_counts/(all_counts*len(trans_counts)))
			sumValue=np.sum(read_counts_frameSum[int(in_excludeLengthParma):])
			sumValue_normed=10**9*(sumValue/(all_counts*len(read_counts_frameSum)))
			NormValue=np.mean(read_counts_frameSum_normed[int(in_excludeLengthParma):])
			if Mode == 'RPKM':
				if NormValue == 0 or sumValue_normed < in_excludeCodonCountsParma:
					filter_5+=1
					continue
			elif Mode == 'counts':
				if NormValue == 0 or sumValue < in_excludeCodonCountsParma:
					filter_5+=1
					continue
			else:
				raise KeyError("There is no such modes, please check it.['RPKM' or 'counts']")
			#get trans density vector
			if Type.upper() == 'CDS':
				(tmpStartWin,tmpStartPos)=getWindowsVector(in_extendRegionLengthParma,in_regionLengthParma,read_counts_frameSum_normed,0) #start codon coor is 0 (0-based)
				(tmpStopWin, tmpStopPos) =getWindowsVector(in_regionLengthParma,in_extendRegionLengthParma,read_counts_frameSum_normed,(len(read_counts_frameSum_normed)-1)) #stop codon coor is len-1 (0-based)
			elif Type.upper() == 'UTR':
				# trans_counts_codon_level=[np.sum(trans_counts[i:i+3]) for i in range(0,len(trans_counts),3)]
				trans_counts_codon_level_normed=[np.sum(trans_counts_normed[i:i+3]) for i in range(0,len(trans_counts_normed),3)]
				(tmpStartWin,tmpStartPos)=getWindowsVector(in_extendRegionLengthParma,in_regionLengthParma,trans_counts_codon_level_normed,int(leftCoor/3)) #start codon coor is 0 (0-based)
				(tmpStopWin, tmpStopPos) =getWindowsVector(in_regionLengthParma,in_extendRegionLengthParma,trans_counts_codon_level_normed,int(rightCoor/3)) #stop codon coor is len-1 (0-based)
			else:
				raise IOError("Please input an correct --type parameters!")
			if norm in ['yes','y','Y','YES','Yes']:
				startNormedWindowsList.append(tmpStartWin/NormValue)
				startWindowsExistPosList.append(tmpStartPos)
				stopNormedWindowsList.append(tmpStopWin/NormValue)
				stopWindowsExistPosList.append(tmpStopPos)
				startNormedWindowsDict[trans]=tmpStartWin/NormValue
				passTransSet.add(trans)
			elif norm in ['no','n','N','NO','No','not','Not','NOT']:
				startNormedWindowsList.append(tmpStartWin)
				startWindowsExistPosList.append(tmpStartPos)
				stopNormedWindowsList.append(tmpStopWin)
				stopWindowsExistPosList.append(tmpStopPos)
				passTransSet.add(trans)
				startNormedWindowsDict[trans]=tmpStartWin
			else:
				raise IOError("Please enter a proper --norm parameter [yes or not]")
	## nt unit
	if Unit=="nt":
		for trans in in_selectTrans:
			i-=1
			leftCoor =int(in_startCodonCoorDict[trans])-1 #the first base of start codon 0-base
			rightCoor=int(in_stopCodonCoorDict[trans])-3 #the first base of stop codon 0-base
			# cdsLength should obey : (1) >= minFilterLength nt unit (2) CDSlength=n (n could be divided by 3)
			if ((rightCoor-leftCoor) < inCDS_lengthFilterParma):
				filter_1+=1
				continue
			if (rightCoor-leftCoor)%3 !=0:
				filter_2+=1
				continue
			# get transNum and trans 5Pos offset counts vector
			(trans_counts,read_counts_frameSum,total_reads,cds_reads)=get_trans_frame_counts(pysamFile, trans, in_readLengths, in_readOffset, in_transLengthDict[trans], leftCoor, rightCoor)
			if total_reads==0 or in_transLengthDict[trans]==0:
				filter_3+=1
				continue
			cds_reads_normed=10**9*(cds_reads/(all_counts*len(read_counts_frameSum)))
			# cds num
			if Mode=="RPKM":
				if cds_reads_normed < inCDS_countsFilterParma:
					filter_4+=1
					continue
			elif Mode=="counts":
				if cds_reads < inCDS_countsFilterParma:
					filter_4+=1
					continue
			else:
				raise KeyError("There is no such modes, please check it.['RPKM' or 'counts']")

			# normalize the counts for per transcript
			trans_counts_normed=10**6*(trans_counts/(all_counts))
			# trans_counts_normed=10**9*(trans_counts/(all_counts*len(trans_counts)))
			NormValue=np.mean(trans_counts_normed[leftCoor:rightCoor][int(in_excludeLengthParma):])
			sumValue=np.sum(trans_counts[leftCoor:rightCoor][int(in_excludeLengthParma):]) # for nt unit
			sumValue_normed=10**9*(sumValue/(all_counts*len(trans_counts[leftCoor:rightCoor])))
			if Mode == 'RPKM':
				if NormValue == 0 or sumValue_normed < in_excludeCodonCountsParma:
					filter_5+=1
					continue
			elif Mode == 'counts':
				if NormValue == 0 or sumValue < in_excludeCodonCountsParma:
					filter_5+=1
					continue
			else:
				raise KeyError("There is no such modes, please check it.['RPKM' or 'counts']")
			if Type.upper() == 'CDS':
				(tmpStartWin,tmpStartPos)=getWindowsVector(in_extendRegionLengthParma,in_regionLengthParma,trans_counts_normed[leftCoor:rightCoor],0) #for nt unit
				(tmpStopWin, tmpStopPos) =getWindowsVector(in_regionLengthParma,in_extendRegionLengthParma,trans_counts_normed[leftCoor:rightCoor],(len(trans_counts_normed[leftCoor:rightCoor])-1)) #for nt unit
			elif Type.upper() == 'UTR':
				(tmpStartWin,tmpStartPos)=getWindowsVector(in_extendRegionLengthParma,in_regionLengthParma,trans_counts_normed,leftCoor) #for nt unit
				(tmpStopWin, tmpStopPos) =getWindowsVector(in_regionLengthParma,in_extendRegionLengthParma,trans_counts_normed,rightCoor) #for nt unit
			else:
				raise IOError("Please input an correct --type parameters!")
			if norm in ['yes','y','Y','YES','Yes']:
				startNormedWindowsList.append(tmpStartWin/NormValue)
				startWindowsExistPosList.append(tmpStartPos)
				stopNormedWindowsList.append(tmpStopWin/NormValue)
				stopWindowsExistPosList.append(tmpStopPos)
				startNormedWindowsDict[trans]=tmpStartWin/NormValue
				passTransSet.add(trans)
			elif norm in ['no','n','N','NO','No','not','Not','NOT']:
				startNormedWindowsList.append(tmpStartWin)
				startWindowsExistPosList.append(tmpStartPos)
				stopNormedWindowsList.append(tmpStopWin)
				stopWindowsExistPosList.append(tmpStopPos)
				passTransSet.add(trans)
				startNormedWindowsDict[trans]=tmpStartWin
			else:
				raise IOError("Please enter a proper --norm parameter [yes or not]")
	###norm per position by mean or median
	startNormedWindowsList=np.array(startNormedWindowsList)
	startWindowsExistPosList=np.array(startWindowsExistPosList)
	stopNormedWindowsList=np.array(stopNormedWindowsList)
	stopWindowsExistPosList=np.array(stopWindowsExistPosList)
	#
	for terms in range(in_extendRegionLengthParma+in_regionLengthParma+1):
		startDensity[terms]=np.mean(startNormedWindowsList[np.where(startWindowsExistPosList[:,terms]==1),terms])
		stopDensity[terms] =np.mean(stopNormedWindowsList[np.where(stopWindowsExistPosList[:,terms]==1),terms])
		start_lower_CI[terms],start_upper_CI[terms]=CI_for_t_distribution(startNormedWindowsList[np.where(startWindowsExistPosList[:,terms]==1),terms][0],confidence=confidence)
		stop_lower_CI[terms],stop_upper_CI[terms]=CI_for_t_distribution(stopNormedWindowsList[np.where(stopWindowsExistPosList[:,terms]==1),terms][0],confidence=confidence)
	pysamFile.close()
	print("Lenght filter(-l)---Transcripts number filtered by criterion one is : "+str(filter_1),file=sys.stderr)
	print("Lenght filter (3n)---Transcripts number filtered by criterion two is : "+str(filter_2),file=sys.stderr)
	print("Total counts filter---Transcripts number filtered by criterion three is : "+str(filter_3),file=sys.stderr)
	print("CDS density filter(RPKM-n or counts-n)---Transcripts number filtered by criterion four is : "+str(filter_4),file=sys.stderr)
	print("CDS density filter(normed-m)---Transcripts number filtered by criterion five is : "+str(filter_5),file=sys.stderr)
	print("Metaplots Transcript Number for bam file"+in_bamFile+" is :"+str(len(passTransSet)),file=sys.stderr)
	return (startDensity,stopDensity,passTransSet,startNormedWindowsDict,start_lower_CI,start_upper_CI,stop_lower_CI,stop_upper_CI)

def metagenePlot(meanDensityDataFrame,in_regionLengthParma,in_extendRegionLengthParma,inOutPrefix,unit):
	plt.rc('font',weight='bold')
	samples=np.unique(meanDensityDataFrame.iloc[:,0].values)
	text_font={"size":30,"family":"Arial","weight":"bold"}
	legend_font={"size":30,"family":"Arial","weight":"bold"}
	winLen=in_regionLengthParma+in_extendRegionLengthParma+1
	if len(samples) <=8:
		colors=["b","orangered","green","c","m","y","k","w"]
	else:
		colors=sns.color_palette('husl',len(samples))
	## start codon
	with PdfPages(inOutPrefix + "_metagenePlot.pdf") as pdf:
		fig1=plt.figure(figsize=(16,8))
		ax1=fig1.add_subplot(111)
		for i in np.arange(len(samples)):
			plt.plot(np.arange(0,winLen),meanDensityDataFrame.iloc[np.where(meanDensityDataFrame.iloc[:,0]==samples[i])].iloc[:,1],color=colors[i],label=samples[i],linewidth=1)
			ax1.set_xticks(np.arange(0,winLen,50))
			ax1.set_xticklabels((np.arange(0,winLen,50)-in_extendRegionLengthParma))
			if unit == 'codon':
				ax1.set_xlabel("Distance from start codon"+' (codon)',fontdict=text_font)
			elif unit == 'nt':
				ax1.set_xlabel("Distance from start codon "+' (nt)',fontdict=text_font)
			else:
				raise KeyError("KeyError:"+str(unit))
			ax1.set_ylabel("Relative footprint density (AU)",fontdict=text_font)
			ax1.set_title("Ribosome footprint density profiles",fontdict=text_font)
			ax1.spines["top"].set_visible(False)
			ax1.spines["right"].set_visible(False)
			ax1.spines["bottom"].set_linewidth(2)
			ax1.spines["left"].set_linewidth(2)
			ax1.tick_params(which="both",width=2,labelsize=20)
			ax1.axhline(1,color="r",dashes=(2,3),clip_on=False,alpha=0.5)
			plt.legend(loc="best",prop=legend_font)
		## stop codon
		fig2=plt.figure(figsize=(16,8))
		ax2=fig2.add_subplot(111)
		for i in np.arange(len(samples)):
			##  stop codon
			plt.plot(np.arange(0,winLen),meanDensityDataFrame.iloc[np.where(meanDensityDataFrame.iloc[:,0]==samples[i])].iloc[:,2],color=colors[i],label=samples[i],linewidth=1)
			ax2.set_xticks(np.arange(0,winLen,50))
			ax2.set_xticklabels((np.arange(0,winLen,50)-in_regionLengthParma))
			if unit == 'codon':
				ax2.set_xlabel("Distance from stop codon"+' (codon)',fontdict=text_font)
			elif unit == 'nt':
				ax2.set_xlabel("Distance from stop codon "+' (nt)',fontdict=text_font)
			else:
				raise KeyError("KeyError:"+str(unit))
			ax2.set_ylabel("Relative footprint density (AU)",fontdict=text_font)
			ax2.set_title("Ribosome footprint density profiles",fontdict=text_font)
			ax2.spines["top"].set_visible(False)
			ax2.spines["right"].set_visible(False)
			ax2.spines["bottom"].set_linewidth(2)
			ax2.spines["left"].set_linewidth(2)
			ax2.tick_params(which="both",width=2,labelsize=20)
			ax2.axhline(1,color="r",dashes=(2,3),clip_on=False,alpha=0.5)
			plt.legend(loc="best",prop=legend_font)
		pdf.savefig(fig1)
		pdf.savefig(fig2)
		plt.close()


def write_passed_transcripts(inBamAttr,outFile):
	data=[]
	data_index=[]
	for bms in inBamAttr:
		trans=bms.passTransSet
		sample=bms.bamLegend
		data.append(trans)
		data_index.append(sample)
	data=pd.DataFrame(data,index=data_index)
	data=data.T
	data.to_csv(outFile,sep="\t",index=0)

def write_codon_units_density(inBamAttr,upstream,downstream,num,outFile,Type):
	try:
		for bms in inBamAttr:
			with open(outFile+"_"+bms.bamLegend+"_codon_density.txt","w") as fout:
				fout.write("%s\t%s\n" % (bms.bamLegend,"start_codon"))
				fout.write("%s" % ("transcript_id"))
				if Type.upper()=="CDS":
					for ss in range(int(num)):
						fout.write("\t%s" % ("codon_"+str(ss+1)))
				elif Type.upper()=="UTR":
					for ss in range(int(num)*2+1):
						fout.write("\t%s" %("codon_"+str(ss-int(num))))
				else:
					raise IOError("Please enter a correct Type [CDS or UTR]")

				fout.write("\n")
				for ts in bms.startNormedWindowsDict:
					fout.write("%s" % ( ts ) ) #the first num codons
					if Type.upper()=="CDS":
						for codons in range(int(num)):
							fout.write("\t%f" % ( bms.startNormedWindowsDict[ts][codons] ) )
					elif Type.upper()=="UTR":
						for nt in range(int(upstream)+int(downstream)+1):
							if nt < (int(upstream)-int(num)) or (nt > (int(upstream)+int(downstream)+1-int(num))):
								pass
							else:
								fout.write("\t%f" %(bms.startNormedWindowsDict[ts][nt]))
					fout.write("\n")
	except IOError:
			print("IOError! Please check your parameters!")

def write_mean_density_dataframe(startMeanDensityDict,stopMeanDensityDict,startLowerCI,startUpperCI,stopLowerCI,stopUpperCI,outFile):
	data=[]
	for sample in startMeanDensityDict:
		k=pd.DataFrame([sample]*len(startMeanDensityDict[sample]))
		startDensity=pd.DataFrame(startMeanDensityDict[sample])
		stopDensity=pd.DataFrame(stopMeanDensityDict[sample])
		startLCI=pd.DataFrame(startLowerCI[sample])
		startUCI=pd.DataFrame(startUpperCI[sample])
		stopLCI=pd.DataFrame(stopLowerCI[sample])
		stopUCI=pd.DataFrame(stopUpperCI[sample])
		density=pd.merge(startDensity,stopDensity,how="left",left_index=True,right_index=True)
		startCI=pd.merge(startLCI,startUCI,how="left",left_index=True,right_index=True)
		stopCI=pd.merge(stopLCI,stopUCI,how="left",left_index=True,right_index=True)
		CI=pd.merge(startCI,stopCI,how="left",left_index=True,right_index=True)
		density=pd.merge(density,CI,how="left",left_index=True,right_index=True)
		density=pd.merge(k,density,how="left",left_index=True,right_index=True)
		data.append(density)
	temp=data[0]
	if len(data) < 1:
		raise EOFError("Empty file, there is nothing in the file.")
	if len(data) == 1:
		temp.columns=['sample','start_density','stop_density','start_lower_CI','start_upper_CI','stop_lower_CI','stop_upper_CI']
		temp.to_csv(outFile,sep="\t",index=0)
		return temp
	else:
		for i in np.arange(1,len(data)):
			temp=np.vstack((temp,data[i]))
		temp=pd.DataFrame(temp,columns=["sample","start_density","stop_density",'start_lower_CI','start_upper_CI','stop_lower_CI','stop_upper_CI'])
		temp.to_csv(outFile,sep="\t",index=0)
		return temp

def parse_args_for_CDS_metagene_analysis():
	parsed=create_parser_for_metagene_analysis()
	(options,args)=parsed.parse_args()
	if options.bamListFile and (options.bam_files or options.read_length or options.read_offset or options.bam_file_legend):
		raise IOError("'-f' parameter and '-i -r -s -t' are mutually exclusive.")
	if options.bamListFile:
		bamFiles,readLengths,Offsets,bamLegends=parse_bamListFile(options.bamListFile)
	elif options.bam_files:
		bamFiles,readLengths,Offsets,bamLegends=options.bam_files.split(","),options.read_length.split("_"),options.read_offset.split("_"),options.bam_file_legend.split(",")
	else:
		raise IOError("Please check you input files!")
	if options.norm_codon_density_num > options.min_cds_codon:
		print("Notice! The -y parameter should not be bigger than -l parameter!")
	if options.norm_codon_density_num > abs(int(options.downstream_codon)+int(options.upstream_codon)):
		raise IOError("Output codon density num bigger than the selective region length [-y > -u + -d]. Reset -y or -u and -d  parameters.")
	if options.type.upper()=="UTR":
		if options.norm_codon_density_num> options.downstream_codon or options.norm_codon_density_num>options.upstream_codon:
			raise IOError("Output codon density num bigger than the selective region length [-y > -u or -y > -d]. Reset -y or -u and -d  parameters.")
	else :
		if options.norm_codon_density_num>options.norm_codon_density_num> options.downstream_codon:
			raise IOError("Output codon density num bigger than the selective region length [-y > -d]. Reset -y or -u and -d  parameters.")

	print("your input : "+ str(len(bamFiles))+" bam files",file=sys.stderr)
	bam_attr=[]
	for ii,jj,mm,nn in zip(bamFiles,readLengths,Offsets,bamLegends):
		bam=bam_file_attr(ii,jj,mm,nn)
		bam_attr.append(bam)
	## calculate density for each bam files
	selectTrans,transLengthDict,startCodonCoorDict,stopCodonCoorDict,transID2geneID,transID2geneName,cdsLengthDict,transID2ChromDict=reload_transcripts_information(options.coorFile)
	geneID2transID={v:k for k,v in transID2geneID.items()}
	geneName2transID={v:k for k,v in transID2geneName.items()}
	if options.in_selectTrans:
		select_trans=pd.read_csv(options.in_selectTrans,sep="\t")
		select_trans=set(select_trans.iloc[:,0].values)
		if options.id_type == 'transcript_id':
			select_trans=select_trans.intersection(selectTrans)
			print("There are " + str(len(select_trans)) + " transcripts from "+options.in_selectTrans+" used for following analysis.",file=sys.stderr)
		elif options.id_type == 'gene_id':
			tmp=[geneID2transID[gene_id] for gene_id in select_trans if gene_id in geneID2transID]
			select_trans=set(tmp)
			select_trans=select_trans.intersection(selectTrans)
			print("There are " + str(len(select_trans))+" gene id could be transformed into transcript id and used for following analysis.",file=sys.stderr)
		elif options.id_type == 'gene_name' or options.id_type=='gene_symbol':
			tmp=[geneName2transID[gene_name] for gene_name in select_trans if gene_name in geneName2transID]
			select_trans=set(tmp)
			select_trans=select_trans.intersection(selectTrans)
			print("There are " + str(len(select_trans))+" gene symbol could be transformed into transcript id and used for following analysis.",file=sys.stderr)
		else:
			raise IOError("Please input a approproate id_type parameters.[transcript_id/gene_id/gene_name/]")
	else:
		select_trans=selectTrans
	startMeanDensityDict={}
	stopMeanDensityDict={}
	startLowerCI={}
	startUpperCI={}
	stopLowerCI={}
	stopUpperCI={}
	for bamfs in bam_attr:
		(bamfs.start_density,bamfs.stop_density,bamfs.passTransSet,bamfs.startNormedWindowsDict,bamfs.start_lower_CI,bamfs.start_upper_CI,bamfs.stop_lower_CI,bamfs.stop_upper_CI) = ribosomeDensityNormPerTrans(bamfs.bamName,select_trans,transLengthDict,
		startCodonCoorDict,stopCodonCoorDict,options.min_cds_codon,options.min_cds_counts,options.downstream_codon,options.upstream_codon,options.norm_exclude_codon,
		options.min_norm_region_counts,bamfs.bamLen,bamfs.bamOffset,options.mode,options.unit,options.type,options.confidence,options.norm)
		print("Finish the step of ribosomeDensityNormPerTrans",file=sys.stderr)
		startMeanDensityDict[bamfs.bamLegend]=bamfs.start_density
		stopMeanDensityDict[bamfs.bamLegend]=bamfs.stop_density
		startLowerCI[bamfs.bamLegend]=bamfs.start_lower_CI
		startUpperCI[bamfs.bamLegend]=bamfs.start_upper_CI
		stopLowerCI[bamfs.bamLegend]=bamfs.stop_lower_CI
		stopUpperCI[bamfs.bamLegend]=bamfs.stop_upper_CI
	## write density
	mean_density_dataframe=write_mean_density_dataframe(startMeanDensityDict,stopMeanDensityDict,startLowerCI,startUpperCI,stopLowerCI,stopUpperCI,options.output_prefix+"_dataframe.txt")
	write_passed_transcripts(bam_attr,options.output_prefix+"_transcript_id.txt")
	write_codon_units_density(bam_attr,options.upstream_codon,options.downstream_codon,options.norm_codon_density_num, options.output_prefix,options.type)
	print("Finish the step of MetageneAnalysis!",file=sys.stderr)
	if options.plot.upper() in ['YES','Y']:
		metagenePlot(mean_density_dataframe,options.downstream_codon,options.upstream_codon,options.output_prefix,options.unit)
		print("Finish the step of metagenePlot!",file=sys.stderr)
	elif options.plot.upper() in ['NO','N','NONE']:
		pass
	else:
		raise IOError("Please input a correct --plot parameter! [yes/no]")


def main():
	"""main program"""
	## parse the options and args and output
	parse_args_for_CDS_metagene_analysis()


if __name__ == "__main__":
		main()





#!/usr/bin/env python
# -*- coding:UTF-8 -*-
'''
@Description: This script is used for statistic the length distribution of sequence reads based on a fastq file.
'''



import sys
import pysam
import numpy as np
from itertools import groupby
from collections import defaultdict
import matplotlib
matplotlib.use('Agg')
import matplotlib.pyplot as plt
from optparse import OptionParser
from .__init__ import __version__


def create_parse_for_plot_reads_length():
	'''argument parser'''
	usage="usage: python %prog [options]"
	parser=OptionParser(usage=usage,version=__version__)
	parser.add_option('-i','--input',action="store",type="string",dest="InputFile",help="Sequence fastq file or bam file.[required]")
	parser.add_option('-o',"--otput_prefix",action="store",type="string",dest="output_prefix",help="Prefix of output files.[required]")
	parser.add_option('-f',"--format",action="store",type="string",dest="format",default='fastq',help="Format of input file, fastq or bam.[required]")
	return parser

def fq2seqDict(fqFile):
	'''
	This function is used to get a dict of transcript sequence based on fastq file
	'''
	fastaDict={}
	f=open(fqFile,'r')
	faiter=(x[1] for x in groupby(f,lambda line: line[0]=="@")) ## groupby returns a tuple (key, group)
	for header in faiter:
		read_name=header.__next__().strip("@").split(" ")[0]
		seq='\t'.join(s.strip() for s in faiter.__next__()).strip().split("\t")[0]
		fastaDict[read_name]=seq
	return fastaDict

def bam2seqDict(bamFile):
	bamDict={}
	f=pysam.AlignmentFile(bamFile)
	for line in f:
		read_name=line.query_name
		read_seq=line.query_sequence
		bamDict[read_name]=read_seq
	return bamDict

def get_read_length(inputFile,Format,output_prefix):
	'''
	This function is used for getting length of all reads from a sequence fastq file.
	'''
	lengths_list=[]
	lengths_dict=defaultdict(int)
	zeros=0
	if Format=='fastq' or Format=='fq':
		read_sequences=fq2seqDict(inputFile)
	elif Format == 'bam' or Format=='BAM':
		read_sequences=bam2seqDict(inputFile)
	else:
		raise IOError("Please reset your -f parameter [fastq or bam]")
	with open(output_prefix+"_reads_length.txt",'w') as f:
		f.write("%s\t%s\n" %("read_name","read_length"))
		for read in read_sequences.keys():
			read_seq=read_sequences[read]
			read_length=len(read_seq)
			if read_length == 0:
				zeros+=1
			lengths_dict[read_length]+=1
			lengths_list.append(read_length)
			f.write("%s\t%s\n" % (str(read),str(read_length)))
	lengths_list=np.array(lengths_list)
	print("The total reads number is: " + str(len(lengths_list)),file=sys.stderr)
	print("The zero length number is: " + str(zeros),file=sys.stderr)
	print("The mean of reads length is: " + str(lengths_list.mean()),file=sys.stderr)
	print("The sd of reads length is: " + str(lengths_list.std()),file=sys.stderr)
	return lengths_list,lengths_dict

def plot_reads_length(lengths_dict,output_prefix,text_font={"size":20,"family":"Arial","weight":"bold"}):
	'''
	plot length distribution
	'''
	labels=lengths_dict.keys()
	values=lengths_dict.values()
	plt.rc('font',weight='bold')
	fig=plt.figure(figsize=(5,4))
	ax=fig.add_subplot(111)
	plt.bar(labels,values,color="b",width=0.5,alpha=0.9)
	ax.spines["top"].set_linewidth(2)
	ax.spines["right"].set_linewidth(2)
	ax.spines["bottom"].set_linewidth(2)
	ax.spines["left"].set_linewidth(2)
	ax.set_xlabel("Length of reads",fontdict=text_font)
	ax.set_ylabel("Read Counts",fontdict=text_font)
	# ax.set_xticks(np.arange(0,50,5))
	# ax.set_xticklabels((np.arange(0,50,5)))
	ax.tick_params(which="both",width=2,labelsize=10)
	plt.tight_layout()
	plt.savefig(output_prefix+"_reads_length.pdf")
	plt.close()

def main():
	parser=create_parse_for_plot_reads_length()
	(options,args)=parser.parse_args()
	if not options.InputFile or not options.output_prefix:
		raise IOError("Please reset your parameters!")
	print("Start the step of length statistics...",file=sys.stderr)
	lengths_list,lengths_dict=get_read_length(options.InputFile,options.format,options.output_prefix)
	plot_reads_length(lengths_dict,options.output_prefix,text_font={"size":15,"family":"Arial","weight":"bold"})
	print("Finish the step of length statistics!",file=sys.stderr)

if __name__=="__main__":
	main()


#!/usr/bin/env python
# -*- coding:UTF-8 -*-
'''
@Author: Li Fajin
@Date: 2019-08-22 16:58:19
@LastEditors: Li Fajin
@LastEditTime: 2019-09-17 15:48:56
@Description: This script is used for statistic reads mapped to DNA  based on bam files mapped to transcriptome.
Becaused the reads mapped to DNA may be contaminations. Code part from Xiao Zhengzao.
'''
import sys
import HTSeq
from collections import Counter
import matplotlib
matplotlib.use("Agg")
import matplotlib.pyplot as plt
from optparse import OptionParser
from .__init__ import __version__

def create_parse_for_DNA_mapped_reads():
	'''argument parser'''
	usage="usage: python %prog [options]"
	parser=OptionParser(usage=usage,version=__version__)
	parser.add_option('-i','--input',action="store",type="string",dest="bamFile",help="Input files mapped to transcriptome with bam format. [required]")
	parser.add_option("-g","--gtfFile",action="store",type="string",dest="gtfFile",help="geome annotation file with gtf format.[required]")
	parser.add_option('-o',"--otput_prefix",action="store",type="string",dest="output_prefix",help="Prefix of output files.[required]")
	return parser

def get_gene_features(gtfFile,id_type,feature_type):
	'''
	get exon features and gene interval features
	'''
	features=HTSeq.GenomicArrayOfSets( "auto", stranded = "yes" )
	geneFeatures=HTSeq.GenomicArrayOfSets( "auto", stranded = "yes" )
	geneRange={}
	gtf=HTSeq.GFF_Reader(gtfFile)
	i=0
	for line in gtf:
		if line.type == feature_type:
			feature_id=line.attr[id_type]
			features[line.iv]+=feature_id
			if feature_id not in geneRange:
				geneRange[feature_id]=[line.iv.chrom,0,0,line.iv.strand]
			if geneRange[feature_id][1] != 0:
				geneRange[feature_id][1]=min(geneRange[feature_id][1],line.iv.start)
			else:
				geneRange[feature_id][1]=line.iv.start
			geneRange[feature_id][2]=max(geneRange[feature_id][2],line.iv.end)
		i+=1
		if i % 100000 == 0:
			print("%d GFF lines processed.\n" % i,file=sys.stderr)
	for g,v in geneRange.items():
		chrom,start,end,strand=v
		tmp_iv=HTSeq.GenomicInterval(chrom,start,end,strand)
		geneFeatures[tmp_iv]+=g
	return features,geneFeatures

def statistic_mapped_reads(bamFile,gtfFile,id_type,feature_type,output_prefix):
	'''
	statistic the reads number mapped to RNA, DNA or introns
	'''
	uniqueGene=0
	uniqueNotGene=0
	uniqueIntron=0
	uniqueAmbiguous=0
	RNA_len=Counter()
	DNA_len=Counter()
	Intron_len=Counter()
	bam=HTSeq.BAM_Reader(bamFile)
	features,geneFeatures=get_gene_features(gtfFile,id_type,feature_type)
	i=0
	for r in bam:
		i+=1
		if i%1000000==0:
			print("%d BAM alignments records processed.\n" % i,file=sys.stderr)
		if not r.aligned:
			continue
		if r.optional_field("NH")==1:
			r.read.seq=r.read_as_aligned.seq
			iv_seq=(co.ref_iv for co in r.cigar if co.type=="M" and co.size >0)
			feature_sets=set()
			intron_featuresets=set()
			for iv in iv_seq:
				for iv2,fs in features[iv].steps():
					feature_sets=feature_sets.union(fs)
				for iv3,fs in geneFeatures[iv].steps():
					intron_featuresets=intron_featuresets.union(fs)
			if feature_sets is None or len(feature_sets) == 0:
				if intron_featuresets is None or len(intron_featuresets)==0:
					uniqueNotGene+=1
					DNA_len[len(r.read.seq)]+=1
				else:
					uniqueIntron+=1
					Intron_len[len(r.read.seq)]+=1
			elif len(feature_sets) ==1:
				uniqueGene+=1
				RNA_len[len(r.read.seq)]+=1
			else:
				uniqueAmbiguous+=1
				RNA_len[len(r.read.seq)]+=1
	with open(output_prefix+"_reads_distribution.txt",'w') as f:
		f.write("unique mapped reads of exon: %i\n" % uniqueGene)
		f.write("unique mapped reads of intergenic region: %i\n" % uniqueNotGene)
		f.write("unique mapped reads of intron: %i\n" % uniqueIntron)
		f.write("unique mapped ambiguous reads of RNA: %i\n" % uniqueAmbiguous)
	return DNA_len,RNA_len,Intron_len

def plot_reads_distribution(lengthDict,output_prefix,text_font={"size":20,"family":"Arial","weight":"bold"}):
	'''
	plot the reads length distribution
	'''
	labels=lengthDict.keys()
	values=lengthDict.values()
	plt.rc('font',weight='bold')
	fig=plt.figure(figsize=(5,4))
	ax=fig.add_subplot(111)
	plt.bar(labels,values,color="b",width=0.5,alpha=0.9)
	ax.spines["top"].set_linewidth(2)
	ax.spines["right"].set_linewidth(2)
	ax.spines["bottom"].set_linewidth(2)
	ax.spines["left"].set_linewidth(2)
	ax.set_xlabel("Length of reads",fontdict=text_font)
	ax.set_ylabel("Read Counts",fontdict=text_font)
	ax.tick_params(which="both",width=2,labelsize=10)
	plt.tight_layout()
	plt.savefig(output_prefix+".pdf")
	plt.close()


def main():
	parser=create_parse_for_DNA_mapped_reads()
	(options,args)=parser.parse_args()
	if not options.bamFile or not options.gtfFile or not options.output_prefix:
		raise IOError("Please reset your parameters!")
	id_type="gene_id"
	feature_type="exon"
	DNA_len,RNA_len,Intron_len=statistic_mapped_reads(options.bamFile,options.gtfFile,id_type,feature_type,options.output_prefix)
	print("Start the step of plot...",file=sys.stderr)
	plot_reads_distribution(DNA_len,options.output_prefix+"_DNA",text_font={"size":15,"family":"Arial","weight":"bold"})
	plot_reads_distribution(RNA_len,options.output_prefix+"_RNA",text_font={"size":15,"family":"Arial","weight":"bold"})
	plot_reads_distribution(Intron_len,options.output_prefix+"_Intron",text_font={"size":15,"family":"Arial","weight":"bold"})
	print("Finish the step Reads statistic!",file=sys.stderr)

if __name__=="__main__":
	main()



#!/usr/bin/env python
# -*- coding:UTF-8 -*-
'''
@Author: Li Fajin
@Date: 2019-08-16 16:50:04
@LastEditors: Li Fajin
@LastEditTime: 2020-05-26 10:50:59
@Description:
This script is used for local tAI index calculation.
input
1) fasta files used for analysis. All files should be split by comma e.g. 1.fasta,2.fasta,3.fasta[required]
2) tRNA gene copy numbers, download from the GtRNAdb
col0: chr
col1: tRNA
col2: start position
col3: stop position
col4: isotype[AA]
col5: Anticodon [required]
col6: upstream sequence
col7: downstream sequence

Notes:
1) The script could output two files: one contains local tAI at each position along transcripts. And the other contains the global tAI.
2) input sequence must be cds sequences which could be generated by GetProteinCodingSequence.py.

'''

from .FunctionDefinition import *
import re
from collections import Counter
from math import pow
from operator import mul
from functools import reduce
from itertools import chain
from itertools import groupby
from scipy.stats import gmean
import Bio.Data.CodonTable as ct


def prepare_datas(tRNA_GCN_file,Codon_table):
	'''According to the GtRNAdb, some anti-codons(tRNA) such as AAA do not have gene copy numbers, so when I construct the codon_anticodon dict,
	 I did not add those codons without copy numbers. Actually, we can add those tRNAs as long as we define the copy numbers equals to 0 at the tRNA_GCN files,
	 in which case, when we calculate the absolute adaptiveness values, we dont have to design the numbers of tRNA, with just only one calculating methods,namely,
	 when len(tRNAs)==2
	 Reference:
	 1) 2004-Mario-tAI
	 2) 2010-Tamir Tuller-Cell
	 '''

	tRNA=pd.read_csv(tRNA_GCN_file,sep='\t')
	codonList=Codon_table.keys()
	tRNA_GCNs=Counter(tRNA['Anticodon'])
	tRNA_GCNs={tRNA.replace("T","U"):tRNA_GCNs[tRNA] for tRNA in tRNA_GCNs.keys()}
	codon_anticodon={
		'UUU':['GAA','AAA'], ## AAA copy number equals to 0
		'UUC':['GAA'],
		'UUA':['UAA'],
		'UUG':['UAA','CAA'],
		'UCU':['AGA','GGA'], ## GGA copy number equals to 0
		'UCC':['AGA','GGA'],
		'UCA':['AGA','UGA'],
		'UCG':['UGA','CGA'],
		'UAU':['AUA','GUA'],
		'UAC':['GUA'],
		'UAA':['UUA'],
		'UAG':['CUA'],
		'UGU':['ACA','GCA'],
		'UGC':['GCA'],
		'UGA':['UCA'],
		'UGG':['CCA'],
		'CUU':['AAG','GAG'], # GAG copy number equals to 0
		'CUC':['AAG','GAG'],
		'CUA':['AAG','UAG'],
		'CUG':['UAG','CAG'],
		'CCU':['AGG','GGG'],
		'CCC':['AGG','GGG'],
		'CCA':['AGG','UGG'],
		'CCG':['UGG','CGG'],
		'CAU':['GUG','AUG'],
		'CAC':['GUG'],
		'CAA':['UUG'],
		'CAG':['UUG','CUG'],
		'CGU':['ACG','GCG'], ## GCG 0
		'CGC':['ACG','GCG'],
		'CGA':['ACG','UCG'],
		'CGG':['UCG','CCG'],
		'AUU':['AAU','GAU'],
		'AUC':['AAU','GAU'],
		'AUA':['AAU','GAU','UAU'],
		'AUG':['CAU'],
		'ACU':['AGU','GGU'],
		'ACC':['AGU','GGU'], ## GGU 0
		'ACA':['AGU','UGU'],
		'ACG':['UGU','CGU'],
		'AAU':['AUU','GUU'],
		'AAC':['GUU'],
		'AAA':['UUU'],
		'AAG':['UUU','CUU'],
		'AGU':['ACU','GCU'],
		'AGC':['GCU'],
		'AGA':['UCU'],
		'AGG':['UCU','CCU'],
		'GUU':['AAC','GAC'], ## GAC 0
		'GUC':['AAC','GAC'],
		'GUA':['AAC','UAC'],
		'GUG':['UAC','CAC'],
		'GCU':['AGC','GGC'], ## GGC 0
		'GCC':['AGC','GGC'],
		'GCA':['AGC','UGC'],
		'GCG':['UGC','CGC'],
		'GAU':['GUC','AUC'], ## AUC 0
		'GAC':['GUC'],
		'GAA':['UUC'],
		'GAG':['UUC','CUC'],
		'GGU':['GCC','ACC'], ## ACC 0
		'GGC':['GCC','ACC'],
		'GGA':['UCC','ACC'],
		'GGG':['UCC','CCC']
	}
	anticodonList=set(list(reduce(chain,codon_anticodon.values())))
	for anticodon in anticodonList:
		if anticodon not in tRNA_GCNs.keys():
			tRNA_GCNs[anticodon]=0
		else:
			pass
	wobble_pair=["I:U","G:C","U:A","C:G","G:U","I:C","I:A","U:G","L:A"] ## A could transfer to I on natural conditions,A:A ==>> I:A
	wobble_pair_Svalues=[0,0,0,0,0.41,0.28,0.999,0.68,0.89] # 0.41(2004)/0.561(2010)
	Sij={k:v for k,v in  zip(wobble_pair,wobble_pair_Svalues)}
	return codonList,tRNA_GCNs,Sij,codon_anticodon

def reverse_complement(seq):
	seq_rc=''.join(["AUCGN"["UAGCN".index(n)] for n in seq[::-1]])
	return seq_rc

def calculate_absolute_adaptiveness_W(codon,GCN,Sij,codon_anticodon):
	anticodon=reverse_complement(codon)
	tRNAs=codon_anticodon[codon]
	if len(tRNAs)==1:
		if tRNAs[0][0]=="A":
			W=(1-Sij["I:U"])*GCN[tRNAs[0]]
		elif tRNAs[0][0]=="G":
			W=(1-Sij["G:C"])*GCN[tRNAs[0]]
		elif tRNAs[0][0]=="U":
			W=(1-Sij["U:A"])*GCN[tRNAs[0]]
		elif tRNAs[0][0]=="C":
			W=(1-Sij["C:G"])*GCN[tRNAs[0]]
		else:
			raise EOFError("Codons error!")
	elif len(tRNAs)==2:
		if codon[-1]=="U" and anticodon[0]=="A":
			W=(1-Sij["I:U"])*GCN[anticodon]+(1-Sij["G:U"])*GCN["G"+anticodon[1:]]
		if codon[-1]=="C" and anticodon[0]=="G":
			W=(1-Sij["G:C"])*GCN[anticodon]+(1-Sij["I:C"])*GCN["A"+anticodon[1:]]
		if codon[-1]=="A" and anticodon[0]=="U":
			W=(1-Sij["U:A"])*GCN[anticodon]+(1-Sij["I:A"])*GCN["A"+anticodon[1:]]
		if codon[-1]=="G" and anticodon[0]=="C":
			W=(1-Sij["C:G"])*GCN[anticodon]+(1-Sij["U:G"])*GCN["U"+anticodon[1:]]
	elif len(tRNAs)==3:
		W=(1-Sij["U:A"])*GCN[anticodon]+(1-Sij["I:A"])*GCN["A"+anticodon[1:]] ## no G:A wooble pairs
	return W

# def calculate_geometric_mean(values):
# 	length=len(values)
# 	return pow(reduce(mul,values),1/length)


def calculate_relative_adaptiveness_w(codonList,GCN,Sij,codon_anticodon):
	w={}
	for codon in codonList:
		codon=codon.replace("T","U")
		W=calculate_absolute_adaptiveness_W(codon,GCN,Sij,codon_anticodon)
		w[codon]=W
	W_max=np.max(list(w.values()))
	W_without_zero=[w for w in w.values() if w !=0]
	W_geometric_mean=gmean(W_without_zero)
	for k,v in w.items():
		if v != 0:
			w[k]=v/W_max
		else:
			w[k]=W_geometric_mean
	return w


def get_trans_frame_tAI(transcriptFile,codonList,GCN,Sij,codon_anticodon,upLength,downLength):
	fastaDict=fastaIter(transcriptFile)
	tAIDict=calculate_relative_adaptiveness_w(codonList,GCN,Sij,codon_anticodon)
	in_selectTrans=fastaDict.keys()
	startAI=np.zeros(int(upLength+downLength+1),dtype='float64')
	stoptAI=np.zeros(int(upLength+downLength+1),dtype='float64')
	startAIList=[]
	stoptAIList=[]
	startPos=[]
	stopPos=[]
	passTransSet=set()
	tAI={}
	tAI_codon={}
	i=len(in_selectTrans)
	for trans in in_selectTrans:
		i-=1
		tmptAI=[]
		cds_seq=fastaDict[trans][:-3] # exclude stop codon
		cds_seq=re.sub("T","U",cds_seq)
		if len(cds_seq)%3 != 0:
			continue
		codon_seq=[cds_seq[i:i+3] for i in np.arange(0,len(cds_seq),3)]
		for codon in codon_seq:
			tmptAI.append(tAIDict[codon])
		# normValue=np.sum(tmptAI)
		# normValue=np.mean(tmptAI) ## huge difference between two normalization methods
		# print("normvalue: ",normValue)
		tAI[trans]=gmean(tmptAI)
		tAI_codon[trans]=tmptAI
		(tmpStartWin,tmpStartPos)=getWindowsVector(upLength,downLength,tmptAI,0) #start codon coor is 0 (0-based), codon level
		(tmpStopWin, tmpStopPos) =getWindowsVector(downLength,upLength,tmptAI,(len(tmptAI)-1))  #stop codon coor is len-1 (0-based) codon level
		startAIList.append(tmpStartWin)
		stoptAIList.append(tmpStopWin)
		startPos.append(tmpStartPos)
		stopPos.append(tmpStopPos)
		passTransSet.add(trans)
	startAIList=np.array(startAIList)
	startPos=np.array(startPos)
	stoptAIList=np.array(stoptAIList)
	stopPos=np.array(stopPos)
	for terms in np.arange(upLength+downLength+1):
		startAI[terms]=np.mean(startAIList[np.where(startPos[:,terms]==1),terms])
		stoptAI[terms] =np.mean(stoptAIList[np.where(stopPos[:,terms]==1),terms])

	print("Metaplots Transcript Number for fasta file"+transcriptFile+" is :"+str(len(passTransSet)),file=sys.stderr)
	return(startAI,stoptAI,tAI,tAI_codon)

def codon2AA(table=1):
	''' table=1 is a standard codon table'''
	codon2AA_dict={k: v.forward_table for k, v in ct.unambiguous_dna_by_id.items()}
	return codon2AA_dict[table]
def get_stop_codons(table=1):
	stop_codons_dict={k: v.stop_codons for k, v in ct.unambiguous_dna_by_id.items()}
	return stop_codons_dict[table]

def write_trans_file_tAI_dataframe(inFastaAttr,outFile):
	data=[]
	for fasta in inFastaAttr:
		k=pd.DataFrame([fasta.fastaLegend]*len(fasta.startAI))
		start=pd.DataFrame(fasta.startAI)
		stop=pd.DataFrame(fasta.stoptAI)
		tAI=pd.merge(start,stop,how="left",left_index=True,right_index=True)
		tAI=pd.merge(k,tAI,how="left",left_index=True,right_index=True)
		data.append(tAI)
	temp=data[0]
	if len(data) < 1:
		raise EOFError("Empty file, there is nothing in the file.")
	if len(data) == 1:
		temp.columns=["sample","start_tAI","stop_tAI"]
		temp.to_csv(outFile,sep="\t",index=0)
	else:
		for i in np.arange(1,len(data)):
			temp=np.vstack((temp,data[i]))
		temp=pd.DataFrame(temp,columns=["sample","start_tAI","stop_tAI"])
		temp.to_csv(outFile,sep="\t",index=0)

def write_tAI_of_each_gene(inFastaAttr,outFile):
	data=[]
	data_index=[]
	for fasta in inFastaAttr:
		d=fasta.tAI
		i=fasta.fastaLegend
		data.append(d)
		data_index.append(i)
	data=pd.DataFrame(data,index=data_index)
	data=data.T
	data.to_csv(outFile,sep="\t")
def write_codon_units_density(inFastaAttr,outFile):
	for fasta in inFastaAttr:
		with open(outFile+"_"+fasta.fastaLegend+"_codon_tAI.txt","w") as fout:
			for ts in fasta.tAI_codon :
				fout.write("%s" % ( ts ) ) #the first num codons
				for codons in range(len(fasta.tAI_codon[ts])):
					fout.write("\t%f" % ( fasta.tAI_codon[ts][codons] ) )
				fout.write("\n")
			#fout.write("\n\n")
def parse_args_for_tAI_calculation():
	parser=create_parser_for_tAI()
	(options,args)=parser.parse_args()
	(transcriptFiles,copy_numbers,outPrefix,trans_file_legend,upLength,downLength,table)=(options.transcriptFiles.split(","),options.copy_numbers,
	options.output_prefix,options.trans_file_legend.split(","),options.upstream_codon,options.downstream_codon,options.geneticCode)
	print("your input : "+ str(len(transcriptFiles))+" transcript files",file=sys.stderr)
	## handle bam file attr
	fasta_attr=[]
	for ii,jj in zip(transcriptFiles,trans_file_legend):
		fasta=fasta_attrbution(ii,jj)
		fasta_attr.append(fasta)
	print("Prepare the files used for tAI calculation...",file=sys.stderr)
	Codon_table=codon2AA(table=table)
	stopCodons=get_stop_codons(table=table)
	for stop_codon in stopCodons:
		Codon_table[stop_codon]="*"
	codonList,tRNA_GCNs,Sij,codon_anticodon=prepare_datas(copy_numbers,Codon_table)
	w=calculate_relative_adaptiveness_w(codonList,tRNA_GCNs,Sij,codon_anticodon)
	w=[[k,v] for k,v in w.items()]
	w=pd.DataFrame(w,columns=['codon','tAI'])
	w.to_csv("tAI_of_each_codon.txt",sep='\t',index=0)
	print("Finish the step of files preparation...",file=sys.stderr)
	print("Start the step of tAI calculations...",file=sys.stderr)

	for fasta in fasta_attr:
		(fasta.startAI,fasta.stoptAI,fasta.tAI,fasta.tAI_codon) = get_trans_frame_tAI(fasta.fastaName,codonList,tRNA_GCNs,Sij,codon_anticodon,upLength,downLength)

	print("Finish the step of get_trans_frame_tAI...",file=sys.stderr)
	write_trans_file_tAI_dataframe(fasta_attr,outPrefix+"_tAI_dataframe.txt")
	write_tAI_of_each_gene(fasta_attr,outPrefix+"_global_tAI.txt")
	write_codon_units_density(fasta_attr,outPrefix)
	print("Finish the step of write_bam_file_density",file=sys.stderr)

def main():
	parse_args_for_tAI_calculation()

if __name__ == "__main__":
	main()



#!/usr/bin/env python
# -*- coding:UTF-8 -*-
'''
Description: This script is used for extract sequence based on a specific position, for example, extract around 5 base centered on codon 50.
For example: 12345N12345
Usage: python ExtractSequenceCenteredOnAPosition.py -i cds_sequence.fa -o output_prefix --center 50 (nt) --stretch 20 (nt)
'''

import numpy as np
import sys
import os
import re
from itertools import groupby
from optparse import OptionParser
import pandas as pd
from .__init__ import __version__

def create_parser():
	usage="python %prog [options]" +"\n" + __doc__+"\n"
	parser=OptionParser(usage=usage,version=__version__)
	parser.add_option("-i",'--fasta-file',action="store",type='string',dest="cdsSequence",
		help="CDS sequences with fasta format. All sequences are from the longest transcript which may be generated by GetProteinCodingSequence.py")
	parser.add_option("--center",action="store",type="int",dest="center_position",default=150,
		help="The center position that we want to fix up. default=%default")
	parser.add_option("--stretch",action="store",type='int',dest="stretch_length",default=3,
		help="The stretch length based on the center position.default=%default.")
	parser.add_option("-o","--output_prefix",action="store",type="string",dest="output_prefix",
		help="Prefix of output files.[required]")
	return parser

def fastaIter(transcriptFile):
	'''
	This function is used to get a dict of transcript sequence
	'''
	fastaDict={}
	f=open(transcriptFile,'r')
	faiter=(x[1] for x in groupby(f,lambda line: line.strip()[0]==">")) ## groupby returns a tuple (key, group)
	for header in faiter:
		geneName=header.__next__().strip(">").split(" ")[0]
		seq=''.join(s.strip() for s in faiter.__next__())
		flag=0
		for nt in ['I','K','M','R','S','W','Y','B','D','H','V','N','X']:
			if nt in seq:
				flag+=1
				flag_nt=nt
		if flag != 0:
			print(geneName+" filtered"+"--"+"There is a ambiguous nucleotide",flag_nt,"in your sequence")
			continue
		fastaDict[geneName]=seq
	return fastaDict


def extract_motif_sequence(transcriptFile,in_selectTrans,center_position,stretch_length,output_prefix):
	tranSeqDict=fastaIter(transcriptFile)
	fout=open(output_prefix+"_centered_sequence.fa",'w')
	for trans in in_selectTrans:
		cds_seq=tranSeqDict[trans][:-3] ## excluding the stop codon.
		if len(cds_seq)%3 != 0:
			continue
		if (center_position+stretch_length) > len(cds_seq):
			raise IOError("--center + --stretch <= length of a cds sequence. Please reset your parameters!")
		tmp_seq=cds_seq[int(center_position-stretch_length-1):int(center_position+stretch_length)]
		fout.write("%s%s\n" %(">",trans))
		fout.write("%s\n" %(str(tmp_seq)))

def main():
	parser=create_parser()
	(options,args)=parser.parse_args()
	(transcriptFile,center_position,stretch_length,output_prefix)=(options.cdsSequence,options.center_position,options.stretch_length,options.output_prefix)
	if not transcriptFile:
		raise IOError("Please enter the cds sequence containing interested transcripts which could be generated by GetProteinCodingSequence.py.")
	if stretch_length> center_position:
		raise IOError("Please the stretch length must be less than centered position. Namely --stretch <= --center")
	tranSeqDict=fastaIter(transcriptFile)
	in_selectTrans=tranSeqDict.keys()
	print("Start extract the sequence...")
	extract_motif_sequence(transcriptFile,in_selectTrans,center_position,stretch_length,output_prefix)
	print("Finish the step of extracting sequences.")


if __name__=="__main__":
	main()




#!/usr/bin/env python
# -*- coding:UTF-8 -*-
'''
@Author: Li Fajin
@Date: 2020-01-07 15:21:12
LastEditors: Li Fajin
LastEditTime: 2020-12-27 20:17:28
@Description: The script is used for plot position depth for each longest transcript.
Usage: python PlotTransCoverage.py -i coverage.txt -o output_prefix -c coorFile -t [transcript_id|gene_id|gene_name] -m [single-gene|gene-list] --id-type [transcript_id|gene_id|gene_name] --color [lightskyblue]
'''

from .FunctionDefinition import *
from .__init__ import __version__
import matplotlib
matplotlib.use('Agg')
import matplotlib.pyplot as plt
import matplotlib.gridspec as gridspec
from matplotlib.backends.backend_pdf import PdfPages
import seaborn as sns
from optparse import OptionParser



def create_parse_for_trans_coverage():
	'''argument parser'''
	usage="usage: python %prog [options]"
	parser=OptionParser(usage=usage,version=__version__)
	parser.add_option('-i','--input',action="store",type="string",dest="InputFile",help="Coverage file generated by CoverageOfEachTrans.py.[required]")
	parser.add_option('-o',"--otput_prefix",action="store",type="string",dest="output_prefix",help="Prefix of output files.[required]")
	parser.add_option("-c","--coordinateFile",action="store",type="string",dest="coorFile",
			help="The file should contain the coordinate of start and stop codon. Generated by OutputTranscriptInfo.py.[required]")
	parser.add_option("-t","--target-gene",action="store",type="string",dest="targetGene",
			help="Target genes for plot. If '-m single-gene', '-t' is a gene name or gene id or transcript id; elif '-m gene-list', '-t' is a file containing some gene names or gene ids or transcript ids with a column name.")
	parser.add_option('--id-type',action="store",type="string",dest="id_type",default="transcript_id",
			help="define the id type users input. the default is transcript id, if not, will be transformed into transcript id. default=%default")
	parser.add_option('--color',action="store",type="string",dest="color",default="lightskyblue",
			help="Color for plot. default=%default")
	parser.add_option("--ymax",action="store",type="float",dest="ymax",default=None,help="The max of ylim. default=%default")
	parser.add_option("--ymin",action="store",type="float",dest="ymin",default=None,help="The min of ylim. default=%default")
	parser.add_option("--type",action="store",type="string",dest="Type",default='single-gene',
			help="Type for input. Either 'single-gene' or 'gene-list'. default=%default.")
	parser.add_option('--mode',action="store",type="string",dest="mode",default="coverage",
			help="Mode of calculation. Either coverage or density of a transcript or transcripts. default=%default")
	return parser

def PlotForSingeGeneCoverage(Coverage,targetTrans,startCoor,stopCoor,trans2GeneDict,color,output_prefix,ymin,ymax):
	singleTransCoverage=Coverage[targetTrans]
	targetTransLength=len(singleTransCoverage)
	label=trans2GeneDict[targetTrans]+":"+targetTrans
	plt.rc("font",family="Arial",weight="bold")
	fig=plt.subplots(figsize=(8,4))
	gs = gridspec.GridSpec(2,1,height_ratios=[11,1],hspace=0.6,left=0.2,right=0.95)
	ax1=plt.subplot(gs[0])
	color=color
	ax1.bar(np.arange(targetTransLength),singleTransCoverage,width=1,facecolor=color)
	# ax1.vlines(np.arange(targetTransLength),ymin=0,ymax=singleTransCoverage,colors=color)
	ax1.spines["top"].set_visible(False)
	ax1.spines["right"].set_visible(False)
	ax1.spines["bottom"].set_linewidth(2)
	ax1.spines["left"].set_linewidth(2)
	ax1.axvline(startCoor,color="gray",dashes=(3,2),alpha=0.5)
	ax1.axvline(stopCoor,color="gray",dashes=(3,2),alpha=0.5)
	ax1.tick_params(which="both",width=2,labelsize=18)
	ax1.set_ylabel("Relative Depth",fontsize=18,fontdict={"size":18,"family":"Arial","weight":"bold"})
	ax1.set_xlim(0,targetTransLength)
	if not ymin and not ymax:
		pass
	elif not ymin and ymax:
		ax1.set_ylim(0,ymax)
	elif ymin and not ymax:
		raise IOError("Please offer the ymax parameter as well!")
	elif ymin and ymax:
		ax1.set_ylim(ymin,ymax)
	else:
		raise IOError("Please enter correct ymin and ymax parameters!")
	plt.title(label,fontdict={"size":18,"family":"Arial","weight":"bold"},loc="center")

	width=0.15
	ax2=plt.subplot(gs[1])
	ax2.set_xlim(0,targetTransLength)
	ax2.fill((startCoor-1,stopCoor,stopCoor,startCoor-1),(1+width/2,1+width/2,1-width/2,1-width/2),color=color,lw=0.5,zorder=20)
	ax2.axhline(1,color='gray',lw=2)
	ax2.set_frame_on(False)
	ax2.xaxis.set_ticks_position("none")
	ax2.yaxis.set_ticks_position("none")
	ax2.set_xticks([])
	ax2.set_yticks([])
	ax2.set_ylim((1-width/2,1+width/2))
	plt.savefig(output_prefix+"_coverage.pdf")

def PlotForSingeGeneDensity(Density,targetTrans,startCoor,stopCoor,trans2GeneDict,color,output_prefix,ymin,ymax):
	singleTransDensity=Density[targetTrans]
	targetTransLength=len(singleTransDensity)
	label=trans2GeneDict[targetTrans]+":"+targetTrans
	colors = sns.color_palette('husl',3)*(1+targetTransLength//3)
	plt.rc("font",family="Arial",weight="bold")
	fig=plt.subplots(figsize=(8,4))
	gs = gridspec.GridSpec(2,1,height_ratios=[11,1],hspace=0.6,left=0.2,right=0.95)
	ax1=plt.subplot(gs[0])
	ax1.vlines(np.arange(targetTransLength),ymin=np.zeros(targetTransLength),ymax=singleTransDensity,colors=colors[:-1],linewidth=2)
	# ax1.bar(np.arange(targetTransLength),singleTransDensity,width=1,facecolor=color)
	# ax1.vlines(np.arange(targetTransLength),ymin=0,ymax=singleTransDensity,colors=color)
	ax1.spines["top"].set_visible(False)
	ax1.spines["right"].set_visible(False)
	ax1.spines["bottom"].set_linewidth(2)
	ax1.spines["left"].set_linewidth(2)
	ax1.axvline(startCoor,color="gray",dashes=(3,2),alpha=0.5)
	ax1.axvline(stopCoor,color="gray",dashes=(3,2),alpha=0.5)
	ax1.tick_params(which="both",width=2,labelsize=18)
	ax1.set_ylabel("Relative Density",fontsize=18,fontdict={"size":18,"family":"Arial","weight":"bold"})
	ax1.set_xlim(0,targetTransLength)
	if not ymin and not ymax:
		pass
	elif not ymin and ymax:
		ax1.set_ylim(0,ymax)
	elif ymin and not ymax:
		raise IOError("Please offer the ymax parameter as well!")
	elif ymin and ymax:
		ax1.set_ylim(ymin,ymax)
	else:
		raise IOError("Please enter correct ymin and ymax parameters!")
	plt.title(label,fontdict={"size":18,"family":"Arial","weight":"bold"},loc="center")

	width=0.15
	ax2=plt.subplot(gs[1])
	ax2.set_xlim(0,targetTransLength)
	ax2.fill((startCoor-1,stopCoor,stopCoor,startCoor-1),(1+width/2,1+width/2,1-width/2,1-width/2),color=color,lw=0.5,zorder=20)
	ax2.axhline(1,color='gray',lw=2)
	ax2.set_frame_on(False)
	ax2.xaxis.set_ticks_position("none")
	ax2.yaxis.set_ticks_position("none")
	ax2.set_xticks([])
	ax2.set_yticks([])
	ax2.set_ylim((1-width/2,1+width/2))
	plt.savefig(output_prefix+"_density.pdf")

def PlotForGeneListsCoverage(Coverage,targetTrans,startCoorDict,stopCoorDict,trans2GeneDict,color,output_prefix,ymin,ymax):
	with PdfPages(output_prefix + "_coverage.pdf") as pdf:
		for trans in targetTrans:
			if trans not in Coverage:
				continue
			plt.rc("font",family="Arial",weight="bold")
			fig=plt.subplots(figsize=(8,4))
			singleTransCoverage=Coverage[trans]
			targetTransLength=len(singleTransCoverage)
			label=trans2GeneDict[trans]+":"+trans
			gs = gridspec.GridSpec(2,1,height_ratios=[11,1],hspace=0.6,left=0.2,right=0.95)
			ax1=plt.subplot(gs[0])
			color=color
			ax1.bar(np.arange(targetTransLength),singleTransCoverage,width=1,facecolor=color)
			# ax1.vlines(np.arange(targetTransLength),ymin=0,ymax=singleTransCoverage,colors=color)
			ax1.spines["top"].set_visible(False)
			ax1.spines["right"].set_visible(False)
			ax1.spines["bottom"].set_linewidth(2)
			ax1.spines["left"].set_linewidth(2)
			ax1.axvline(startCoorDict[trans],color="gray",dashes=(3,2),alpha=0.5)
			ax1.axvline(stopCoorDict[trans],color="gray",dashes=(3,2),alpha=0.5)
			ax1.tick_params(which="both",width=2,labelsize=18)
			ax1.set_ylabel("Relative Depth",fontsize=18,fontdict={"size":18,"family":"Arial","weight":"bold"})
			ax1.set_xlim(0,targetTransLength)
			if not ymin and not ymax:
				pass
			elif not ymin and ymax:
				ax1.set_ylim(0,ymax)
			elif ymin and not ymax:
				raise IOError("Please offer the ymax parameter as well!")
			elif ymin and ymax:
				ax1.set_ylim(ymin,ymax)
			else:
				raise IOError("Please enter correct ymin and ymax parameters!")
			plt.title(label,fontdict={"size":18,"family":"Arial","weight":"bold"},loc="center")

			width=0.15
			ax2=plt.subplot(gs[1])
			ax2.set_xlim(0,targetTransLength)
			ax2.fill((startCoorDict[trans]-1,stopCoorDict[trans],stopCoorDict[trans],startCoorDict[trans]-1),(1+width/2,1+width/2,1-width/2,1-width/2),color=color,lw=0.5,zorder=20)
			ax2.axhline(1,color='gray',lw=2)
			ax2.set_frame_on(False)
			ax2.xaxis.set_ticks_position("none")
			ax2.yaxis.set_ticks_position("none")
			ax2.set_xticks([])
			ax2.set_yticks([])
			ax2.set_ylim((1-width/2,1+width/2))
			pdf.savefig()
		plt.close()

def PlotForGeneListsDensity(Density,targetTrans,startCoorDict,stopCoorDict,trans2GeneDict,color,output_prefix,ymin,ymax):
	with PdfPages(output_prefix + "_density.pdf") as pdf:
		for trans in targetTrans:
			if trans not in Density:
				continue
			plt.rc("font",family="Arial",weight="bold")
			fig=plt.subplots(figsize=(8,4))
			singleTransDensity=Density[trans]
			targetTransLength=len(singleTransDensity)
			label=trans2GeneDict[trans]+":"+trans
			gs = gridspec.GridSpec(2,1,height_ratios=[11,1],hspace=0.6,left=0.2,right=0.95)
			ax1=plt.subplot(gs[0])
			colors = sns.color_palette('husl',3)*(1+targetTransLength//3)
			ax1.vlines(np.arange(targetTransLength),ymin=np.zeros(targetTransLength),ymax=singleTransDensity,colors=colors[:-1],linewidth=2)
			# ax1.vlines(np.arange(targetTransLength),ymin=0,ymax=singleTransDensity,colors=color)
			ax1.spines["top"].set_visible(False)
			ax1.spines["right"].set_visible(False)
			ax1.spines["bottom"].set_linewidth(2)
			ax1.spines["left"].set_linewidth(2)
			ax1.axvline(startCoorDict[trans],color="gray",dashes=(3,2),alpha=0.5)
			ax1.axvline(stopCoorDict[trans],color="gray",dashes=(3,2),alpha=0.5)
			ax1.tick_params(which="both",width=2,labelsize=18)
			ax1.set_ylabel("Relative Density",fontsize=18,fontdict={"size":18,"family":"Arial","weight":"bold"})
			ax1.set_xlim(0,targetTransLength)
			if not ymin and not ymax:
				pass
			elif not ymin and ymax:
				ax1.set_ylim(0,ymax)
			elif ymin and not ymax:
				raise IOError("Please offer the ymax parameter as well!")
			elif ymin and ymax:
				ax1.set_ylim(ymin,ymax)
			else:
				raise IOError("Please enter correct ymin and ymax parameters!")
			plt.title(label,fontdict={"size":18,"family":"Arial","weight":"bold"},loc="center")

			width=0.15
			ax2=plt.subplot(gs[1])
			ax2.set_xlim(0,targetTransLength)
			ax2.fill((startCoorDict[trans]-1,stopCoorDict[trans],stopCoorDict[trans],startCoorDict[trans]-1),(1+width/2,1+width/2,1-width/2,1-width/2),color=color,lw=0.5,zorder=20)
			ax2.axhline(1,color='gray',lw=2)
			ax2.set_frame_on(False)
			ax2.xaxis.set_ticks_position("none")
			ax2.yaxis.set_ticks_position("none")
			ax2.set_xticks([])
			ax2.set_yticks([])
			ax2.set_ylim((1-width/2,1+width/2))
			pdf.savefig()
		plt.close()

def read_coverage(Coverage):
	coverageDict={}
	with open(Coverage,'r') as f:
		for line in f:
			trans_id=line.strip().split("\t")[0]
			depth=np.array([float(i) for i in line.strip().split("\t")[1:]])
			coverageDict[trans_id]=depth
	return coverageDict

def parse_for_trans_coverage():
	parser=create_parse_for_trans_coverage()
	(options,args)=parser.parse_args()
	(InputFile,output_prefix,coorFile,Type,targetGene,id_type,color,ymin,ymax)=(options.InputFile,options.output_prefix,options.coorFile,options.Type,
    options.targetGene,options.id_type,options.color,options.ymin,options.ymax)
	CoverageDict=read_coverage(InputFile)
	selectTrans,transLengthDict,startCodonCoorDict,stopCodonCoorDict,transID2geneID,transID2geneName,cdsLengthDict,transID2ChromDict=reload_transcripts_information(coorFile)
	geneID2transID={v:k for k,v in transID2geneID.items()}
	geneName2transID={v:k for k,v in transID2geneName.items()}
	if Type == 'single-gene':
		if id_type=='transcript_id':
			targetTrans=targetGene
		elif id_type=='gene_id':
			try:
				targetTrans=geneID2transID[targetGene]
			except IOError:
				print("There is no such target gene in annotation file!")
		elif id_type=='gene_name':
			try:
				targetTrans=geneName2transID[targetGene]
			except IOError:
				print("There is no such target gene in annotation file!")
		else:
			raise IOError("Please input a approproate id_type parameters.[transcript_id/gene_id/gene_name/]")
		if targetTrans  not in CoverageDict:
			raise IOError("There is no such "+targetGene +" in input file.")
		print("Start plot...",file=sys.stderr)
		if options.mode.strip().upper() in ["COVERAGE","C"]:
			PlotForSingeGeneCoverage(CoverageDict,targetTrans,startCodonCoorDict[targetTrans],stopCodonCoorDict[targetTrans],transID2geneName,color,output_prefix,ymin,ymax)
		elif options.mode.strip().upper() in ["DENSITY","COUNTS","COUNT","D"]:
			PlotForSingeGeneDensity(CoverageDict,targetTrans,startCodonCoorDict[targetTrans],stopCodonCoorDict[targetTrans],transID2geneName,color,output_prefix,ymin,ymax)
		else:
			raise IOError("Please reset your --mode parameter! [coverage/c or density/d]")
		print("Finish!",file=sys.stderr)
	elif Type == 'gene-list':
		targetTrans=pd.read_csv(targetGene,sep="\t")
		targetTrans=set(targetTrans.iloc[:,0].values)
		if id_type=='transcript_id':
			pass
		elif id_type=='gene_id':
			targetTrans=set([geneID2transID[gene] for gene in targetTrans if gene in geneID2transID.keys()])
		elif id_type=='gene_name':
			targetTrans=set([geneName2transID[gene] for gene in targetTrans if gene in geneName2transID.keys()])
		else:
			raise IOError("Please input a approproate id_type parameters.[transcript_id/gene_id/gene_name/]")
		print("Start plot...",file=sys.stderr)
		if options.mode.strip().upper() in ["COVERAGE","C"]:
			PlotForGeneListsCoverage(CoverageDict,targetTrans,startCodonCoorDict,stopCodonCoorDict,transID2geneName,color,output_prefix,ymin,ymax)
		elif options.mode.strip().upper() in ["DENSITY","COUNTS","COUNT","D"]:
			PlotForGeneListsDensity(CoverageDict,targetTrans,startCodonCoorDict,stopCodonCoorDict,transID2geneName,color,output_prefix,ymin,ymax)
		else:
			raise IOError("Please reset your --mode parameter! [coverage/c or density/d]")
		print("Finish!",file=sys.stderr)
	else:
		raise IOError("Please enter a correct mode [single-gene or gene-list]")

def main():
	parse_for_trans_coverage()


if __name__=="__main__":
	main()


#!/usr/bin/env python
# -*- coding:UTF-8 -*-
'''
@Author: Li Fajin
@Date: 2019-08-16 15:09:23
LastEditors: Li Fajin
LastEditTime: 2020-12-27 15:08:06
@Description: The script is used for calculating ribosome density at each position along transcript and as well as "coverage" of each transcript.
'''


from .FunctionDefinition import *



def ribosomeDensityAtEachPosition(in_bamFile,in_bamLegend,in_selectTrans,in_transLengthDict,in_startCodonCoorDict,in_stopCodonCoorDict,in_readLengths,in_readOffset,Unit,output_prefix):
	pysamFile=pysam.AlignmentFile(in_bamFile,"rb")
	pysamFile_trans=pysamFile.references
	in_selectTrans=set(pysamFile_trans).intersection(in_selectTrans).intersection(in_startCodonCoorDict.keys())
	## codon unit
	if Unit=="codon":
		with open(output_prefix+"_"+in_bamLegend.strip()+"_cds_codon_density.txt",'w') as f1,open(output_prefix+"_"+in_bamLegend.strip()+"_cds_codon_coverage.txt",'w') as f2:
			f1.write("%s\t%s\n" %("transcript",in_bamLegend))
			f2.write("%s\t%s\n" %("transcript",in_bamLegend))
			for trans in in_selectTrans:
				leftCoor =int(in_startCodonCoorDict[trans])-1 #the first base of start codon 0-base
				rightCoor=int(in_stopCodonCoorDict[trans])-3 #the first base of stop codon 0-base
				(trans_counts,read_counts_frameSum,total_reads,cds_reads)=get_trans_frame_counts(pysamFile, trans, in_readLengths, in_readOffset, in_transLengthDict[trans], leftCoor, rightCoor)
				cds_codon_density=np.array(read_counts_frameSum)
				cds_coverage=np.sum(cds_codon_density!=0)/len(read_counts_frameSum)
				f2.write("%s\t%s\n" %(trans,str(cds_coverage)))
				f1.write("%s\t" %(trans))
				for i in range(len(read_counts_frameSum)):
					# print(trans,str(i+1),str(read_counts_frameSum[i]))
					# f1.write("%s\t%s\t%s\n" %(trans,str(i+1),str(read_counts_frameSum[i])))
					f1.write("%s\t" %(str(read_counts_frameSum[i])))
				f1.write("\n")
	elif Unit=="nt":
		with open(output_prefix+"_"+in_bamLegend.strip()+"_cds_nt_density.txt",'w') as f1, open(output_prefix+"_"+in_bamLegend.strip()+"_cds_nt_coverage.txt",'w') as f2:
			f1.write("%s\t%s\n" %("transcript",in_bamLegend))
			f2.write("%s\t%s\n" %("transcript",in_bamLegend))
			for trans in in_selectTrans:
				leftCoor =int(in_startCodonCoorDict[trans])-1 #the first base of start codon 0-base
				rightCoor=int(in_stopCodonCoorDict[trans])-3 #the first base of stop codon 0-base
				(trans_counts,read_counts_frameSum,total_reads,cds_reads)=get_trans_frame_counts(pysamFile, trans, in_readLengths, in_readOffset, in_transLengthDict[trans], leftCoor, rightCoor)
				trans_counts_cds=np.array(trans_counts[leftCoor:rightCoor+3])
				cds_coverage=sum(trans_counts_cds!=0)/len(trans_counts_cds)
				f2.write("%s\t%s\n" %(trans,str(cds_coverage)))
				f1.write("%s\t" %(trans))
				for i in range(len(trans_counts_cds)):
					# print(trans,str(i+1),str(trans_counts[i]))
					# f1.write("%s\t%s\t%s\n" %(trans,str(i+1),str(trans_counts_cds[i])))
					f1.write("%s\t" %(str(trans_counts_cds[i])))
				f1.write("\n")
	else:
		raise IOError("Please enter a proper -U parameter![codon or nt]")

def parse_args_for_riboDensity_atEachPosition():
	parsed=create_parser_for_riboDensity_atEachPosition()
	(options,args)=parsed.parse_args()
	if options.bamListFile and (options.bam_files or options.read_length or options.read_offset or options.bam_file_legend):
		raise IOError("'-f' parameter and '-i -r -s -t' are mutually exclusive.")
	if options.bamListFile:
		bamFiles,readLengths,Offsets,bamLegends=parse_bamListFile(options.bamListFile)
	elif options.bam_files:
		bamFiles,readLengths,Offsets,bamLegends=options.bam_files.split(","),options.read_length.split("_"),options.read_offset.split("_"),options.bam_file_legend.split(",")
	else:
		raise IOError("Please check you input files!")
	print("your input : "+ str(len(bamFiles))+" bam files",file=sys.stderr)
	bam_attr=[]
	for ii,jj,mm,nn in zip(bamFiles,readLengths,Offsets,bamLegends):
		bam=bam_file_attr(ii,jj,mm,nn)
		bam_attr.append(bam)
	selectTrans,transLengthDict,startCodonCoorDict,stopCodonCoorDict,transID2geneID,transID2geneName,cdsLengthDict,transID2ChromDict=reload_transcripts_information(options.coorFile)
	geneID2transID={v:k for k,v in transID2geneID.items()}
	geneName2transID={v:k for k,v in transID2geneName.items()}
	if options.in_selectTrans:
		select_trans=pd.read_csv(options.in_selectTrans,sep="\t")
		select_trans=set(select_trans.iloc[:,0].values)
		if options.id_type == 'transcript_id':
			select_trans=select_trans.intersection(selectTrans)
			print("There are " + str(len(select_trans)) + " transcripts from "+options.in_selectTrans+" used for following analysis.",file=sys.stderr)
		elif options.id_type == 'gene_id':
			tmp=[geneID2transID[gene_id] for gene_id in select_trans if gene_id in geneID2transID]
			select_trans=set(tmp)
			select_trans=select_trans.intersection(selectTrans)
			print("There are " + str(len(select_trans))+" gene id could be transformed into transcript id and used for following analysis.",file=sys.stderr)
		elif options.id_type == 'gene_name' or options.id_type=='gene_symbol':
			tmp=[geneName2transID[gene_name] for gene_name in select_trans if gene_name in geneName2transID]
			select_trans=set(tmp)
			select_trans=select_trans.intersection(selectTrans)
			print("There are " + str(len(select_trans))+" gene symbol could be transformed into transcript id and used for following analysis.",file=sys.stderr)
		else:
			raise IOError("Please input a approproate id_type parameters.[transcript_id/gene_id/gene_name/]")
	else:
		select_trans=selectTrans
	print("There are "+str(len(select_trans))+" transcripts with both start and stop codon will be used for following analysis.",file=sys.stderr)
	select_trans=sorted(list(select_trans))
	for bamfs in bam_attr:
		ribosomeDensityAtEachPosition(bamfs.bamName,bamfs.bamLegend,select_trans,transLengthDict,startCodonCoorDict,stopCodonCoorDict,bamfs.bamLen,bamfs.bamOffset,options.unit,options.output_prefix)
	print("Finish the step of ribosomeDensityAtEachPosition",file=sys.stderr)

def main():
	"""main program"""
	## parse the options and args and output
	parse_args_for_riboDensity_atEachPosition()


if __name__ == "__main__":
	main()





#!/usr/bin/env python
# -*- coding:UTF-8 -*-
'''
@Author: Li Fajin
@Date: 2019-08-18 21:19:13
LastEditors: Li Fajin
LastEditTime: 2020-08-31 18:44:58
@Description: This script is used for calculating RPFdist vaules of each transcript
RPFdist=(read counts in 5UTR)/(read counts in CDS region) or
RPFdist=(density in 5UTR)/(density in CDS region)
'''


from .FunctionDefinition import *


def RPFdist(in_bamFile,in_selectTrans,in_transLengthDict,in_startCodonCoorDict,in_stopCodonCoorDict,inCDS_lengthFilterParma,inCDS_countsFilterParma,in_excludeLengthParma,in_excludeCodonCountsParma,in_readLengths,in_readOffset,Mode):

	pysamFile=pysam.AlignmentFile(in_bamFile,"rb")
	pysamFile_trans=pysamFile.references
	in_selectTrans=set(pysamFile_trans).intersection(in_selectTrans).intersection(in_startCodonCoorDict.keys())
	all_counts=0
	filter_1=0
	filter_2=0
	filter_3=0
	filter_4=0
	filter_5=0
	RPFdist_dict={}
	passTransSet=set()
	for trans in in_startCodonCoorDict.keys():
		leftCoor =int(in_startCodonCoorDict[trans])-1
		rightCoor=int(in_stopCodonCoorDict[trans])-3
		(trans_counts,read_counts_frameSum,total_reads,cds_reads)=get_trans_frame_counts(pysamFile, trans, in_readLengths, in_readOffset, in_transLengthDict[trans], leftCoor, rightCoor)
		all_counts+=total_reads ## total_reads for transcript level

	for trans in in_selectTrans:
		leftCoor =int(in_startCodonCoorDict[trans])-1 #the first base of start codon 0-base
		rightCoor=int(in_stopCodonCoorDict[trans])-3 #the first base of stop codon 0-base
		# cdsLength should obey : (1) >= minFilterLength codon unit (2) CDSlength=3n
		if ((rightCoor-leftCoor) < inCDS_lengthFilterParma*3):
			filter_1+=1
			continue
		if (rightCoor-leftCoor)%3 !=0:
			filter_2+=1
			continue
		# get transNum and trans 5Pos offset counts vector
		(trans_counts,read_counts_frameSum,total_reads,cds_reads)=get_trans_frame_counts(pysamFile, trans, in_readLengths, in_readOffset, in_transLengthDict[trans], leftCoor, rightCoor)
		if total_reads==0 or in_transLengthDict[trans]==0:
			filter_3+=1
			continue
		cds_reads_normed=10**9*(cds_reads/(all_counts*len(read_counts_frameSum)))
		# cds num
		if Mode=="RPKM":
			if cds_reads_normed < inCDS_countsFilterParma:
				filter_4+=1
				continue
		elif Mode=="counts":
			if cds_reads < inCDS_countsFilterParma:
				filter_4+=1
				continue
		else:
			raise KeyError("There is no such modes, please check it.['RPKM' or 'counts']")

		# normalize the counts for per transcript
		normValue=np.mean(read_counts_frameSum[int(in_excludeLengthParma):])
		sumValue=np.sum(read_counts_frameSum[int(in_excludeLengthParma):])
		sumValue_normed=10**9*(sumValue/(all_counts*len(read_counts_frameSum)))
		if Mode == 'RPKM':
			if normValue == 0 or sumValue_normed < in_excludeCodonCountsParma:
				filter_5+=1
				continue
		elif Mode == 'counts':
			if normValue == 0 or sumValue < in_excludeCodonCountsParma:
				filter_5+=1
				continue
		else:
			raise KeyError("There is no such modes, please check it.['RPKM' or 'counts']")
		Five_UTR_counts=trans_counts[:leftCoor]
		CDS_counts=trans_counts[leftCoor:rightCoor]
		trans_counts_normed=10**9*(trans_counts/(all_counts*len(trans_counts)))
		Five_UTR_counts_normed=trans_counts_normed[:leftCoor]
		CDS_counts_normed=trans_counts_normed[leftCoor:rightCoor]
		if len(Five_UTR_counts_normed)==0 or len(Five_UTR_counts)==0:
			continue
		if Mode == 'RPKM':
			if np.sum(CDS_counts_normed)==0:
				RPFdist_dict[trans]=(np.sum(Five_UTR_counts_normed)+1)/(np.sum(CDS_counts_normed)+1)
			else:
				RPFdist_dict[trans]=np.sum(Five_UTR_counts_normed)/np.sum(CDS_counts_normed)
			passTransSet.add(trans)
		elif Mode == 'counts':
			if np.sum(CDS_counts)==0:
				RPFdist_dict[trans]=(np.sum(Five_UTR_counts)+1)/(np.sum(CDS_counts)+1)
			else:
				RPFdist_dict[trans]=np.sum(Five_UTR_counts)/np.sum(CDS_counts)
			passTransSet.add(trans)
		else:
			raise KeyError("There is no such modes, please check it.['RPKM' or 'counts']")
	pysamFile.close()
	print("Lenght filter(-l)---Transcripts number filtered by criterion one is : "+str(filter_1),file=sys.stderr)
	print("Lenght filter (3n)---Transcripts number filtered by criterion two is : "+str(filter_2),file=sys.stderr)
	print("Total counts filter---Transcripts number filtered by criterion three is : "+str(filter_3),file=sys.stderr)
	print("CDS density filter(RPKM-n or counts-n)---Transcripts number filtered by criterion four is : "+str(filter_4),file=sys.stderr)
	print("CDS density filter(normed-m)---Transcripts number filtered by criterion five is : "+str(filter_5),file=sys.stderr)
	print("Metaplots Transcript Number for bam file"+in_bamFile+" is :"+str(len(passTransSet)),file=sys.stderr)
	return RPFdist_dict

def write_bam_file_RPFdist_dataframe(inBamAttr,outFile):
	data=[]
	data_index=[]
	for bms in inBamAttr:
		d=bms.RPFdist_dict
		i=bms.bamLegend
		data.append(d)
		data_index.append(i)
	data=pd.DataFrame(data,index=data_index)
	data=data.T
	data.to_csv(outFile,sep="\t")


def main():
	parsed=create_parser_for_RPFdist()
	(options,args)=parsed.parse_args()
	if options.bamListFile and (options.bam_files or options.read_length or options.read_offset or options.bam_file_legend):
		raise IOError("'-f' parameter and '-i -r -s -t' are mutually exclusive.")
	if options.bamListFile:
		bamFiles,readLengths,Offsets,bamLegends=parse_bamListFile(options.bamListFile)
	elif options.bam_files:
		bamFiles,readLengths,Offsets,bamLegends=options.bam_files.split(","),options.read_length.split("_"),options.read_offset.split("_"),options.bam_file_legend.split(",")
	else:
		raise IOError("Please check you input files!")
	print("your input : "+ str(len(bamFiles))+" bam files",file=sys.stderr)
	bam_attr=[]
	for ii,jj,mm,nn in zip(bamFiles,readLengths,Offsets,bamLegends):
		bam=bam_file_attr(ii,jj,mm,nn)
		bam_attr.append(bam)
	## calculate density for each bam files
	selectTrans,transLengthDict,startCodonCoorDict,stopCodonCoorDict,transID2geneID,transID2geneName,cdsLengthDict,transID2ChromDict=reload_transcripts_information(options.coorFile)
	geneID2transID={v:k for k,v in transID2geneID.items()}
	geneName2transID={v:k for k,v in transID2geneName.items()}
	if options.in_selectTrans:
		select_trans=pd.read_csv(options.in_selectTrans,sep="\t")
		select_trans=set(select_trans.iloc[:,0].values)
		if options.id_type == 'transcript_id':
			select_trans=select_trans.intersection(selectTrans)
			print("There are " + str(len(select_trans)) + " transcripts from "+options.in_selectTrans+" used for following analysis.",file=sys.stderr)
		elif options.id_type == 'gene_id':
			tmp=[geneID2transID[gene_id] for gene_id in select_trans if gene_id in geneID2transID]
			select_trans=set(tmp)
			select_trans=select_trans.intersection(selectTrans)
			print("There are " + str(len(select_trans))+" gene id could be transformed into transcript id and used for following analysis.",file=sys.stderr)
		elif options.id_type == 'gene_name' or options.id_type=='gene_symbol':
			tmp=[geneName2transID[gene_name] for gene_name in select_trans if gene_name in geneName2transID]
			select_trans=set(tmp)
			select_trans=select_trans.intersection(selectTrans)
			print("There are " + str(len(select_trans))+" gene symbol could be transformed into transcript id and used for following analysis.",file=sys.stderr)
		else:
			raise IOError("Please input a approproate id_type parameters.[transcript_id/gene_id/gene_name/]")
	else:
		select_trans=selectTrans

	for bamfs in bam_attr:
		(bamfs.RPFdist_dict) = RPFdist(bamfs.bamName,select_trans,transLengthDict,startCodonCoorDict,stopCodonCoorDict,options.min_cds_codon,
		options.min_cds_counts,options.norm_exclude_codon,options.min_norm_region_counts,bamfs.bamLen,bamfs.bamOffset,options.mode)
		print("Finish the step of RPFdist calculation",file=sys.stderr)
	## write density
	write_bam_file_RPFdist_dataframe(bam_attr,options.output_prefix+"_RPFdist.txt")
	print("Finish the step of write_bam_file_RPFdist_dataframe",file=sys.stderr)

if __name__ == "__main__":
    main()


#!/usr/bin/env python
# -*- coding:UTF-8 -*-
'''
@Author: Li Fajin
@Date: 2020-01-07 10:26:31
LastEditors: Li Fajin
LastEditTime: 2020-12-27 17:09:09
@Description: This script is used for statistic of coverage for each transcript.
'''



import pysam
import pysamstats
import numpy as np
from .FunctionDefinition import *
from .__init__ import __version__



def create_parser_for_coverage():
	'''argument parser.'''
	usage="usage: python %prog [options]" + '\n' + __doc__ + "\n"
	parser=OptionParser(usage=usage,version=__version__)
	parser.add_option("-f","--bamListFile",action="store",type="string",default=None,dest="bamListFile",
			help="Bam file list, containing 4 columns.Namely bamFiles,readLength, offSet, bamLegend. '-f' and '-i, -r, -s, -t' parameters are mutually exclusive.default=%default.")
	parser.add_option("-i","--input", action="store",type="string",default=None,dest="bam_files",
			help="Input file(s) in bam format. All files should be split by comma e.g. 1.bam,2.bam,3.bam[required]. '-i' and '-f' are mutually exclusive. default=%default")
	parser.add_option("-c","--coordinateFile",action="store",type="string",dest="coorFile",
			help="The file should contain the coordinate of start and stop codon. Generated by OutputTranscriptInfo.py.[required]")
	parser.add_option("-o","--otput_prefix",action="store",type="string",dest="output_prefix",default=None,
			help="Prefix of output files.[required],default=%default")
	parser.add_option("-r","--specific_reads_length",action="store",type="string",dest="read_length",
			help="Specific the lenght to do analysis, comma split. e.g. '28,29,30'.If use all length set 'All'. Bam files diff length select split by '_' e.g. '28,29,30_ALL_27,28' [required]. '-r' and '-f' are mutually exclusive.")
	parser.add_option("-s","--offset",action="store",type="string",dest="read_offset",
			help="Specific the offset corresponding to read length, comma split. e.g. '12,13,13'. No offset set 0. Bam files diff offset select split by '_' e.g. '12,13,13_0_12,12' [required]. '-s' and '-f' are mutually exclusive.")
	parser.add_option("-t","--bam_file_legend",action="store",type="string",dest="bam_file_legend",
			help="The legend of each bam files, comma split. e.g. 'condition1,condition2,condition3' [required]. '-t' and '-f' are mutually exclusive.")
	parser.add_option('-S','--select_trans_list',action="store",type='string',dest='in_selectTrans',default=None,
			help="Selected transcript list used for metagene analysis.This files requires the first column must be the transcript ID  with a column name.")
	parser.add_option('--id-type',action="store",type="string",dest="id_type",default="transcript_id",
			help="define the id type users input. the default is transcript id, if not, will be transformed into transcript id. default=%default")
	parser.add_option('--mode',action="store",type="string",dest="mode",default="coverage",
			help="Mode of calculation. Either coverage or density of a transcript or transcripts. default=%default")
	return parser

def CalculateCoverage(in_bamFile,in_bamLegend,in_selectTrans,in_transLengthDict,in_startCodonCoorDict,in_stopCodonCoorDict,in_readLengths,in_readOffset,output_prefix):
	pysamFile=pysam.AlignmentFile(in_bamFile,"rb")
	pysamFile_trans=pysamFile.references
	in_selectTrans=set(pysamFile_trans).intersection(in_selectTrans)
	trans_set=set()
	all_counts=0
	if output_prefix:
		outputFileName=output_prefix+"_"+in_bamLegend
	else:
		outputFileName=in_bamLegend
	for trans in in_startCodonCoorDict.keys():
		leftCoor =int(in_startCodonCoorDict[trans])-1
		rightCoor=int(in_stopCodonCoorDict[trans])-3
		(trans_counts,read_counts_frameSum,total_reads,cds_reads)=get_trans_frame_counts(pysamFile, trans, in_readLengths, in_readOffset, in_transLengthDict[trans], leftCoor, rightCoor)
		all_counts+=total_reads ## total_reads for transcript level

	with open(outputFileName+"_raw_depth.txt",'w') as f1, open(outputFileName+"_RPM_depth.txt",'w') as f2:
		for trans in in_selectTrans:
			# print(trans)
			tmpTrans=pysamstats.load_coverage(pysamFile,chrom=trans,pad=True)
			tmpTransRaw=np.array([i[2] for i in tmpTrans])
			tmpTransRPM=10**6*(tmpTransRaw/all_counts)
			trans_set.add(trans)
			f1.write("%s\t" %(trans))
			f2.write("%s\t" %(trans))
			for i in range(len(tmpTrans)):
				f1.write("%s\t" %(str(tmpTransRaw[i])))
				f2.write("%s\t" %(str(tmpTransRPM[i])))
			f1.write("\n")
			f2.write("\n")
		print("There are about " + str(len(trans_set)) +" transcripts used for coverage calculation!", file=sys.stderr)

def CalculateDensity(in_bamFile,in_bamLegend,in_selectTrans,in_transLengthDict,in_startCodonCoorDict,in_stopCodonCoorDict,in_readLengths,in_readOffset,output_prefix):
	pysamFile=pysam.AlignmentFile(in_bamFile,"rb")
	pysamFile_trans=pysamFile.references
	in_selectTrans=set(pysamFile_trans).intersection(in_selectTrans)
	trans_set=set()
	all_counts=0
	if output_prefix:
		outputFileName=output_prefix+"_"+in_bamLegend
	else:
		outputFileName=in_bamLegend
	for trans in in_startCodonCoorDict.keys():
		leftCoor =int(in_startCodonCoorDict[trans])-1
		rightCoor=int(in_stopCodonCoorDict[trans])-3
		(trans_counts,read_counts_frameSum,total_reads,cds_reads)=get_trans_frame_counts(pysamFile, trans, in_readLengths, in_readOffset, in_transLengthDict[trans], leftCoor, rightCoor)
		all_counts+=total_reads ## total_reads for transcript level

	with open(outputFileName+"_raw_density.txt",'w') as f1, open(outputFileName+"_RPM_density.txt",'w') as f2:
		for trans in in_selectTrans:
			leftCoor =int(in_startCodonCoorDict[trans])-1
			rightCoor=int(in_stopCodonCoorDict[trans])-3
			# print(trans)
			(trans_counts,read_counts_frameSum,total_reads,cds_reads)=get_trans_frame_counts(pysamFile, trans, in_readLengths, in_readOffset, in_transLengthDict[trans], leftCoor, rightCoor)
			tmpTransRaw=np.array(trans_counts)
			tmpTransRPM=10**6*(tmpTransRaw/all_counts)
			trans_set.add(trans)
			f1.write("%s\t" %(trans))
			f2.write("%s\t" %(trans))
			for i in range(len(trans_counts)):
				f1.write("%s\t" %(str(tmpTransRaw[i])))
				f2.write("%s\t" %(str(tmpTransRPM[i])))
			f1.write("\n")
			f2.write("\n")
		print("There are about " + str(len(trans_set)) +" transcripts used for ribosome density calculation!", file=sys.stderr)

def IDtransForm(in_selectTrans,coorFile,id_type):
	selectTrans,transLengthDict,startCodonCoorDict,stopCodonCoorDict,transID2geneID,transID2geneName,cdsLengthDict,transID2ChromDict=reload_transcripts_information(coorFile)
	geneID2transID={v:k for k,v in transID2geneID.items()}
	geneName2transID={v:k for k,v in transID2geneName.items()}
	if in_selectTrans:
		select_trans=pd.read_csv(in_selectTrans,sep="\t")
		select_trans=set(select_trans.iloc[:,0].values)
		if id_type == 'transcript_id':
			select_trans=select_trans.intersection(selectTrans)
			print("There are " + str(len(select_trans)) + " transcripts from "+in_selectTrans+" used for following analysis.",file=sys.stderr)
		elif id_type == 'gene_id':
			tmp=[geneID2transID[gene_id] for gene_id in select_trans if gene_id in geneID2transID]
			select_trans=set(tmp)
			select_trans=select_trans.intersection(selectTrans)
			print("There are " + str(len(select_trans))+" gene id could be transformed into transcript id and used for following analysis.",file=sys.stderr)
		elif id_type == 'gene_name' or id_type=='gene_symbol':
			tmp=[geneName2transID[gene_name] for gene_name in select_trans if gene_name in geneName2transID]
			select_trans=set(tmp)
			select_trans=select_trans.intersection(selectTrans)
			print("There are " + str(len(select_trans))+" gene symbol could be transformed into transcript id and used for following analysis.",file=sys.stderr)
		else:
			raise IOError("Please input a approproate id_type parameters.[transcript_id/gene_id/gene_name/]")
	else:
		select_trans=selectTrans

	return select_trans,transLengthDict,startCodonCoorDict,stopCodonCoorDict

def main():
	parser=create_parser_for_coverage()
	(options,args)=parser.parse_args()
	if options.bamListFile and (options.bam_files or options.read_length or options.read_offset or options.bam_file_legend):
		raise IOError("'-f' parameter and '-i -r -s -t' are mutually exclusive.")
	if options.bamListFile:
		bamFiles,readLengths,Offsets,bamLegends=parse_bamListFile(options.bamListFile)
	elif options.bam_files:
		bamFiles,readLengths,Offsets,bamLegends=options.bam_files.split(","),options.read_length.split("_"),options.read_offset.split("_"),options.bam_file_legend.split(",")
	else:
		raise IOError("Please check you input files!")

	print("your input : "+ str(len(bamFiles))+" bam files",file=sys.stderr)
	bam_attr=[]
	for ii,jj,mm,nn in zip(bamFiles,readLengths,Offsets,bamLegends):
		bam=bam_file_attr(ii,jj,mm,nn)
		bam_attr.append(bam)
	select_trans,transLengthDict,startCodonCoorDict,stopCodonCoorDict=IDtransForm(options.in_selectTrans,options.coorFile,options.id_type)
	for bamfs in bam_attr:
		if options.mode.strip().upper() in ["COVERAGE","C"]:
			CalculateCoverage(bamfs.bamName,bamfs.bamLegend,select_trans,transLengthDict,startCodonCoorDict,stopCodonCoorDict,bamfs.bamLen,bamfs.bamOffset,options.output_prefix)
		elif options.mode.strip().upper() in ["DENSITY","COUNTS","COUNT","D"]:
			CalculateDensity(bamfs.bamName,bamfs.bamLegend,select_trans,transLengthDict,startCodonCoorDict,stopCodonCoorDict,bamfs.bamLen,bamfs.bamOffset,options.output_prefix)
		else:
			raise IOError("Please reset your --mode parameter! [coverage/c or density/d]")


if __name__=="__main__":
    main()



#!/usr/bin/env python
# -*- coding:UTF-8 -*-
'''
@Author: Li Fajin
@Date: 2019-08-27 21:13:08
@LastEditors: Li Fajin
@LastEditTime: 2020-06-08 10:31:23

'''

from functools import reduce
from .FunctionDefinition import *
from operator import mul
from scipy import stats
from collections import defaultdict


def get_density_dict(densityFile):
	density_dict={}
	with open(densityFile,'r') as f:
		i=0
		for line in f:
			i+=1
			if i==1:
				continue
			tmp=line.strip().split("\t")
			trans_id=tmp[0]
			density_dict[trans_id]=[float(i) for i in tmp[1:]]
	return density_dict

def get_all_counts(density_dict):
	all_counts=0
	for trans,counts in density_dict.items():
		all_counts+=sum(counts)
	return all_counts
def CI_for_t_distribution(data,confidence=0.95):
	'''
	scripts source:
	https://www.jianshu.com/p/6cfce4cc2f7f
	'''
	mean=np.mean(data)
	std=np.std(data,ddof=1)
	size=len(data)
	alpha=1-confidence
	t_score=stats.t.isf(alpha/2,df=(size-1))
	ME=t_score*std/np.sqrt(size)
	lower_CI=mean-ME
	upper_CI=mean+ME
	return lower_CI,upper_CI

def enrichment_ratio(ctrl_dict,treat_dict,in_selectTrans,inCDS_lengthFilterParma,inCDS_countsFilterParma,in_regionLengthParma,in_extendRegionLengthParma,in_excludeLengthParma,in_excludeCodonCountsParma,mode,unit,confidence):
	ctrl_all_counts=get_all_counts(ctrl_dict)
	treat_all_counts=get_all_counts(treat_dict)
	in_selectTrans=in_selectTrans.intersection(ctrl_dict.keys()).intersection(treat_dict.keys())
	passTransSet=set()
	ratio_dict={}
	startratio=np.zeros(int(in_regionLengthParma+in_extendRegionLengthParma+1),dtype="float64")
	stopratio =np.zeros(int(in_regionLengthParma+in_extendRegionLengthParma+1),dtype="float64")
	start_lower_CI=np.zeros(int(in_regionLengthParma+in_extendRegionLengthParma+1),dtype="float64")
	start_upper_CI=np.zeros(int(in_regionLengthParma+in_extendRegionLengthParma+1),dtype="float64")
	stop_lower_CI=np.zeros(int(in_regionLengthParma+in_extendRegionLengthParma+1),dtype="float64")
	stop_upper_CI=np.zeros(int(in_regionLengthParma+in_extendRegionLengthParma+1),dtype="float64")
	startNormedWindowsList=[]
	startWindowsExistPosList=[]
	stopNormedWindowsList=[]
	stopWindowsExistPosList=[]
	filter_1=0
	filter_2=0
	filter_3=0
	filter_4=0
	filter_5=0
	for trans in in_selectTrans:
		ctrl_trans_counts=np.array(ctrl_dict[trans])
		treat_trans_counts=np.array(treat_dict[trans])
		cds_length=len(ctrl_trans_counts)
		if unit == 'codon':
			if cds_length < inCDS_lengthFilterParma:
				filter_1+=1
				continue
			if (cds_length*3) % 3 != 0:
				filter_2+=1
				continue
		elif unit == 'nt':
			if cds_length < inCDS_lengthFilterParma:
				filter_1+=1
				continue
			if cds_length % 3 != 0:
				filter_2+=1
				continue
		else:
			raise KeyError("There is no such unit, please check it.['codon' or 'nt']")
		ctrl_trans_counts_rpm=10**6*(ctrl_trans_counts/ctrl_all_counts)
		treat_trans_counts_rpm=10**6*(treat_trans_counts/treat_all_counts)
		ctrl_trans_counts_sum=sum(ctrl_trans_counts)
		treat_trans_counts_sum=sum(treat_trans_counts)
		ctrl_trans_counts_sum_norm=10**9*(ctrl_trans_counts_sum/(ctrl_all_counts*cds_length))
		treat_trans_counts_sum_norm=10**9*(treat_trans_counts_sum/(treat_all_counts*cds_length))
		if ctrl_trans_counts_sum == 0 or treat_trans_counts_sum ==0:
			filter_3+=1
			continue
		if mode == 'RPKM':
			if ctrl_trans_counts_sum_norm < inCDS_countsFilterParma or treat_trans_counts_sum_norm < inCDS_countsFilterParma:
				filter_4+=1
				continue
		elif mode == 'counts':
			if ctrl_trans_counts_sum < inCDS_countsFilterParma or treat_trans_counts_sum < inCDS_countsFilterParma:
				filter_4+=1
				continue
		else:
			raise KeyError("There is no such modes, please check it.['RPKM' or 'counts']")
		ctrl_normValue=np.mean(ctrl_trans_counts_rpm[in_excludeLengthParma:])
		treat_normValue=np.mean(treat_trans_counts_rpm[in_excludeLengthParma:])
		ctrl_sumValue=sum(ctrl_trans_counts[in_excludeLengthParma:])
		treat_sumValue=sum(treat_trans_counts[in_excludeLengthParma:])
		ctrl_sumValue_norm=10**9*(ctrl_sumValue/(ctrl_all_counts*cds_length))
		treat_sumValue_norm=10**9*(treat_sumValue/(treat_all_counts*cds_length))
		if mode == 'RPKM':
			if ctrl_normValue==0 or treat_normValue==0 or ctrl_sumValue_norm <in_excludeCodonCountsParma or treat_sumValue_norm <in_excludeCodonCountsParma:
				filter_5+=1
				continue
		elif mode == 'counts':
			if ctrl_normValue==0 or treat_normValue==0 or ctrl_sumValue <in_excludeCodonCountsParma or treat_sumValue <in_excludeCodonCountsParma:
				filter_5+=1
				continue
		else:
			raise KeyError("There is no such modes, please check it.['RPKM' or 'counts']")
		ratio=np.zeros(cds_length,dtype="float64")
		for i in np.arange(len(ratio)):
			ctrl=ctrl_trans_counts_rpm[i]+1 ## ctrl=ctrl+1
			treat=treat_trans_counts_rpm[i]+1 ## treat=treat+1
			# if ctrl ==0 or treat ==0:
			# 	ctrl+=1
			# 	treat+=1
			ratio[i]+=treat/ctrl
		####get trans density vector
		(tmpStartWin,tmpStartPos)=getWindowsVector(in_extendRegionLengthParma,in_regionLengthParma,ratio,0) #start codon coor is 0 (0-based)
		(tmpStopWin, tmpStopPos) =getWindowsVector(in_regionLengthParma,in_extendRegionLengthParma,ratio,(len(ratio)-1)) #stop codon coor is len-1 (0-based)
		startNormedWindowsList.append(tmpStartWin)
		startWindowsExistPosList.append(tmpStartPos)
		stopNormedWindowsList.append(tmpStopWin)
		stopWindowsExistPosList.append(tmpStopPos)
		passTransSet.add(trans)
		ratio_dict[trans]=ratio

	startNormedWindowsList=np.array(startNormedWindowsList)
	startWindowsExistPosList=np.array(startWindowsExistPosList)
	stopNormedWindowsList=np.array(stopNormedWindowsList)
	stopWindowsExistPosList=np.array(stopWindowsExistPosList)
	for terms in range(in_extendRegionLengthParma+in_regionLengthParma+1):
		startratio[terms]=np.mean(startNormedWindowsList[np.where(startWindowsExistPosList[:,terms]==1),terms])
		stopratio[terms] =np.mean(stopNormedWindowsList[np.where(stopWindowsExistPosList[:,terms]==1),terms])
		if len(in_selectTrans) > 1:
			start_lower_CI[terms],start_upper_CI[terms]=CI_for_t_distribution(startNormedWindowsList[np.where(startWindowsExistPosList[:,terms]==1),terms][0],confidence=confidence)
			stop_lower_CI[terms],stop_upper_CI[terms]=CI_for_t_distribution(stopNormedWindowsList[np.where(stopWindowsExistPosList[:,terms]==1),terms][0],confidence=confidence)
		else:
			pass
	print("filter_1: Lenght filter(-l)---Transcripts number filtered by criterion one is : "+str(filter_1),file=sys.stderr)
	print("filter_2: Lenght filter (3n)---Transcripts number filtered by criterion two is : "+str(filter_2),file=sys.stderr)
	print("filter_3: Trans counts filter (total counts=0)---Transcripts number filtered by criterion three is : "+str(filter_3),file=sys.stderr)
	print("filter_4: CDS counts filter(RPKM-n)---Transcripts number filtered by criterion four is : "+str(filter_4),file=sys.stderr)
	print("filter_5: RPKM(-m) ---Transcripts number filtered by criterion five is : "+str(filter_5),file=sys.stderr)
	print("The number of genes used for following analysis is: " + str(len(passTransSet)),file=sys.stderr)
	if len(in_selectTrans) > 1:
		return (startratio,stopratio,passTransSet,ratio_dict,start_lower_CI,start_upper_CI,stop_lower_CI,stop_upper_CI)
	else:
		return (startratio,stopratio,passTransSet,ratio_dict)


def write_enrichment_transcripts(passTransSet,output_prefix):
	data=[]
	data_index=[]
	trans=passTransSet
	sample=output_prefix
	data.append(trans)
	data_index.append(sample)
	data=pd.DataFrame(data,index=data_index)
	data=data.T
	data.to_csv(output_prefix+"_enrichment_transcript_id.txt",sep="\t",index=0)

def write_mean_density_dataframe(startMeanDensityDict,stopMeanDensityDict,startLowerCI,startUpperCI,stopLowerCI,stopUpperCI,outFile):
	data=[]
	for sample in startMeanDensityDict:
		k=pd.DataFrame([sample]*len(startMeanDensityDict[sample]))
		startDensity=pd.DataFrame(startMeanDensityDict[sample])
		stopDensity=pd.DataFrame(stopMeanDensityDict[sample])
		startLCI=pd.DataFrame(startLowerCI[sample])
		startUCI=pd.DataFrame(startUpperCI[sample])
		stopLCI=pd.DataFrame(stopLowerCI[sample])
		stopUCI=pd.DataFrame(stopUpperCI[sample])
		density=pd.merge(startDensity,stopDensity,how="left",left_index=True,right_index=True)
		startCI=pd.merge(startLCI,startUCI,how="left",left_index=True,right_index=True)
		stopCI=pd.merge(stopLCI,stopUCI,how="left",left_index=True,right_index=True)
		CI=pd.merge(startCI,stopCI,how="left",left_index=True,right_index=True)
		density=pd.merge(density,CI,how="left",left_index=True,right_index=True)
		density=pd.merge(k,density,how="left",left_index=True,right_index=True)
		data.append(density)
	temp=data[0]
	if len(data) < 1:
		raise EOFError("Empty file, there is nothing in the file.")
	if len(data) == 1:
		temp.columns=['sample','start_density','stop_density','start_lower_CI','start_upper_CI','stop_lower_CI','stop_upper_CI']
		temp.to_csv(outFile,sep="\t",index=0)
	else:
		for i in np.arange(1,len(data)):
			temp=np.vstack((temp,data[i]))
		temp=pd.DataFrame(temp,columns=["sample","start_density","stop_density",'start_lower_CI','start_upper_CI','stop_lower_CI','stop_upper_CI'])
		temp.to_csv(outFile,sep="\t",index=0)
def write_enrichment_dataframe(startratio,stopratio,output_prefix):
		data=[]
		k=pd.DataFrame([output_prefix]*len(startratio))
		start=pd.DataFrame(startratio)
		stop=pd.DataFrame(stopratio)
		density=pd.merge(start,stop,how="left",left_index=True,right_index=True)
		density=pd.merge(k,density,how="left",left_index=True,right_index=True)
		data.append(density)
		temp=data[0]
		if len(data) < 1:
				raise EOFError("Empty file, there is nothing in the file.")
		if len(data) == 1:
				temp.columns=['sample','start_density','stop_density']
				temp.to_csv(output_prefix+"_enrichment_dataframe.txt",sep="\t",index=0)
		else:
				for i in np.arange(1,len(data)):
						temp=np.vstack((temp,data[i]))
				temp=pd.DataFrame(temp,columns=["sample","start_density","stop_density"])
				temp.to_csv(output_prefix+"_enrichment_dataframe.txt",sep="\t",index=0)

def write_ratio_dict(ratio_dict,output_prefix):
	with open(output_prefix+"_codon_ratio.txt",'w') as f:
		f.write("%s\t%s\n" %("transcript",output_prefix))
		for trans,ratio in ratio_dict.items():
			f.write("%s\t" % trans)
			for i in range(len(ratio)):
				f.write("%s\t" % str(ratio[i]))
			f.write("\n")


def main():
	parser=create_parser_for_enrichment_analysis()
	(options,args)=parser.parse_args()
	(ctrlDensity,treatDensity,output_prefix, unit,mode,min_cds_codon, min_cds_counts,
	min_norm_region_counts, upstream_codon, downstream_codon, norm_exclude_codon,confidence) = (options.ctrlDensity,options.treatDensity,options.output_prefix,
	options.unit,options.mode,options.min_cds_codon,options.min_cds_counts,options.min_norm_region_counts,options.upstream_codon,
	options.downstream_codon,options.norm_exclude_codon,options.confidence)
	if not ctrlDensity or not treatDensity:
		raise IOError("Please set your --ctrl and --treat parameters!")
	ctrl_dict=get_density_dict(ctrlDensity)
	treat_dict=get_density_dict(treatDensity)
	selectTrans=set(ctrl_dict.keys()).intersection(set(treat_dict))
	## trans id transformation
	transID2geneID,transID2geneName=reload_transcripts_information(options.coorFile)[4:6]
	geneID2transID={v:k for k,v in transID2geneID.items()}
	geneName2transID={v:k for k,v in transID2geneName.items()}
	if options.in_selectTrans:
		select_trans=pd.read_csv(options.in_selectTrans,sep="\t")
		select_trans=set(select_trans.iloc[:,0].values)
		if options.id_type == 'transcript_id':
			select_trans=select_trans.intersection(selectTrans)
			print("There are " + str(len(select_trans)) + " transcripts from "+options.in_selectTrans+" used for following analysis.",file=sys.stderr)
		elif options.id_type == 'gene_id':
			tmp=[geneID2transID[gene_id] for gene_id in select_trans if gene_id in geneID2transID]
			select_trans=set(tmp)
			select_trans=select_trans.intersection(selectTrans)
			print("There are " + str(len(select_trans))+" gene id could be transformed into transcript id and used for following analysis.",file=sys.stderr)
		elif options.id_type == 'gene_name' or options.id_type=='gene_symbol':
			tmp=[geneName2transID[gene_name] for gene_name in select_trans if gene_name in geneName2transID]
			select_trans=set(tmp)
			select_trans=select_trans.intersection(selectTrans)
			print("There are " + str(len(select_trans))+" gene symbol could be transformed into transcript id and used for following analysis.",file=sys.stderr)
		else:
			raise IOError("Please input a approproate id_type parameters.[transcript_id/gene_id/gene_name/]")
	else:
		select_trans=selectTrans
	print("There are "+str(len(select_trans))+" transcripts with both start and stop codon will be used for following analysis.",file=sys.stderr)
	if len(select_trans) > 1:
		startMeanDensityDict={}
		stopMeanDensityDict={}
		startLowerCI={}
		startUpperCI={}
		stopLowerCI={}
		stopUpperCI={}
		(startratio,stopratio,passTransSet,ratio_dict,start_lower_CI,start_upper_CI,stop_lower_CI,stop_upper_CI) = enrichment_ratio(ctrl_dict,treat_dict,select_trans,
		min_cds_codon,min_cds_counts,downstream_codon,upstream_codon,norm_exclude_codon,min_norm_region_counts,mode,unit,confidence)
		startMeanDensityDict[output_prefix]=startratio
		stopMeanDensityDict[output_prefix]=stopratio
		startLowerCI[output_prefix]=start_lower_CI
		startUpperCI[output_prefix]=start_upper_CI
		stopLowerCI[output_prefix]=stop_lower_CI
		stopUpperCI[output_prefix]=stop_upper_CI
		print("Finish the step of enrichment ratio calculation",file=sys.stderr)
		## write density
		write_mean_density_dataframe(startMeanDensityDict,stopMeanDensityDict,startLowerCI,startUpperCI,stopLowerCI,stopUpperCI,output_prefix+"_enrichment_dataframe.txt")
	else:
		(startratio,stopratio,passTransSet,ratio_dict) =enrichment_ratio(ctrl_dict,treat_dict,select_trans,
		min_cds_codon,min_cds_counts,downstream_codon,upstream_codon,norm_exclude_codon,min_norm_region_counts,mode,unit,confidence)
		print("Finish the step of enrichment ratio calculation",file=sys.stderr)
		## write density
		write_enrichment_dataframe(startratio,stopratio,output_prefix)
	write_enrichment_transcripts(passTransSet,output_prefix)
	write_ratio_dict(ratio_dict,output_prefix)
	print("Finish the step of enrichment analysis!",file=sys.stderr)

if __name__=="__main__":
	main()


#!/usr/bin/env python
# -*- coding:UTF-8 -*-
'''

@Description: This script is used for plotting the GC content generated by GCContent.py
'''


import pandas as pd
import sys
import matplotlib
matplotlib.use('Agg')
import matplotlib.pyplot as plt
import seaborn as sns
from optparse import OptionParser
from .__init__ import __version__

def create_parse_for_plot_GC_content():
	'''argument parser'''
	usage="usage: python %prog [options]"
	parser=OptionParser(usage=usage,version=__version__)
	parser.add_option('-i','--input',action="store",type="string",dest="GC_content",help="Input file generated by GCContent.py.[required]")
	parser.add_option('-o',"--otput_prefix",action="store",type="string",dest="output_prefix",help="Prefix of output files.[required]")
	parser.add_option("--mode",action="store",type="string",dest="mode",default="normal",help="The type of GC content you want to statistic. Either the normal type or GC content from each reading frame. [normal or frames]. defaul=%default")
	return parser

def plot_GC_content_for_different_frames(GC_content_melt,output_prefix,text_font={"size":20,"family":"Arial","weight":"bold"}):
	''' plot GC content for different frames'''
	plt.rc('font',weight='bold')
	fig=plt.figure(figsize=(8,6))
	ax=fig.add_subplot(111)
	ax=sns.violinplot(x='variable',y='value',data=GC_content_melt)
	ax.set_xlabel("Different reading frames",fontdict=text_font)
	ax.set_ylabel("GC%",fontdict=text_font)
	ax.spines["top"].set_visible(False)
	ax.spines["right"].set_visible(False)
	ax.spines["bottom"].set_linewidth(2)
	ax.spines["left"].set_linewidth(2)
	ax.tick_params(which="both",width=2,labelsize=10)
	plt.savefig(output_prefix+"_frames_GC_contents.pdf")
	plt.close()

def plot_normal_GC_content(GC_content,output_prefix,text_font={"size":20,"family":"Arial","weight":"bold"}):
	plt.rc('font',weight='bold')
	fig=plt.figure(figsize=(8,6))
	ax=fig.add_subplot(111)
	ax=sns.distplot(GC_content['GC%'],bins=100,kde_kws={"color": "r", "lw": 3},hist_kws={"linewidth": 3,"alpha": 0.5, "color": "b"})
	ax.set_xlabel("GC%",fontdict=text_font)
	ax.set_ylabel("Frequency",fontdict=text_font)
	ax.spines["top"].set_visible(False)
	ax.spines["right"].set_visible(False)
	ax.spines["bottom"].set_linewidth(2)
	ax.spines["left"].set_linewidth(2)
	ax.tick_params(which="both",width=2,labelsize=10)
	plt.savefig(output_prefix+"_GC_contents.pdf")
	plt.close()

def main():
	parser=create_parse_for_plot_GC_content()
	(options,args)=parser.parse_args()
	if not options.GC_content or not options.output_prefix:
		raise IOError("Please your input GC_content file and prefix of your output files.")
	GC_content=pd.read_csv(options.GC_content,sep="\t")
	if options.mode == 'normal':
		print("Start plot GC content...",file=sys.stderr)
		plot_normal_GC_content(GC_content,options.output_prefix,text_font={"size":20,"family":"Arial","weight":"bold"})
		print("Finish the step of plot GC content!",file=sys.stderr)
	elif options.mode == 'frames':
		GC_content_melt=pd.melt(GC_content,id_vars="transcripts")
		print("Start plot GC content...",file=sys.stderr)
		plot_GC_content_for_different_frames(GC_content_melt,options.output_prefix,text_font={"size":20,"family":"Arial","weight":"bold"})
		print("Finish the step of plot GC content!",file=sys.stderr)
	else:
		raise IOError("Please reset your --mode parameter [normal/frames]")

if __name__=="__main__":
	main()


#!/usr/bin/env python
# -*- coding:UTF-8 -*-
'''
In whole mode:
This script is used for getting transcript sequences, cds sequences and amino acid sequence  from protein coding genes.
In local mode:
This script is used for getting a local cds sequence with a specific region [left_pos to right_pos in codon unit]
input:
1) transcripts_sequences.fa: all transcript sequences generated by RiboCode.
2) longest.trans.info.txt: coordinate of the longest cds sequences for protein coding genes generated by OutputTranscriptInfo.py.

output:
1) longest_transcripts_cds.fa: transcript sequences of protein genes in DNA level.
3) longest_transcripts_cds_ORFs.fa: cds sequences of protein coding genes in DNA level.
5) longest_transcripts_aa.fa: amino acid sequences of protein coding genes. Translated from transcripts_cds_DNA.fa.
'''

from .FunctionDefinition import *


def extract_protein_coding_sequence(transcriptFile,startCodonCoorDict,stopCodonCoorDict,in_selectTrans,output_prefix,table):
	'''
	This function is used for extacting sequences of protein coding genes based on transcript_sequences.fa generated by RiboCode.
	'''
	trans_sequence_dict=fastaIter(transcriptFile)
	in_selectTrans=in_selectTrans.intersection(trans_sequence_dict.keys()).intersection(startCodonCoorDict.keys())
	i=0
	with open(output_prefix+"_transcript_sequences.fa",'w') as f1,open(output_prefix+"_cds_sequences.fa",'w') as f2,open(output_prefix+"_amino_acid_sequences.fa",'w') as f3:
		for trans in in_selectTrans:
			trans_sequence=trans_sequence_dict[trans]
			startCoor=int(startCodonCoorDict[trans])-1 # 0-based
			stopCoor=int(stopCodonCoorDict[trans])-3 # 0-based , the first base of stop codon
			cds_sequence=trans_sequence[startCoor:(stopCoor+3)]
			AA_sequence=translation(cds_sequence,table=table,cds=False)
			AA_sequence_length=len(AA_sequence)
			trans_sequence_length=len(trans_sequence)
			cds_sequence_length=len(cds_sequence)
			if cds_sequence_length%3 !=0:
				i+=1
			f1.write("%s%s\n" %(">",str(trans)+" "+str(trans_sequence_length)))
			f1.write("%s\n" %(trans_sequence))
			f2.write("%s%s\n" %(">",str(trans)+" "+str(cds_sequence_length)))
			f2.write("%s\n" %(cds_sequence))
			f3.write("%s%s\n" %(">",str(trans)+" "+str(AA_sequence_length)))
			f3.write("%s\n" %(AA_sequence))
	print("Notes: There are " + str(i) +" transcripts whose cds sequence cannot be divided by 3!",file=sys.stderr)
def extract_local_cds_sequence(transcriptFile,startCodonCoorDict,stopCodonCoorDict,in_selectTrans,output_prefix,left_pos,right_pos,table):
	'''
	This script is used for extracting a local sequence of a given cds sequence. And you have to offer a left position and right position you wanna to extract.
	And the position is in codon unit.
	'''
	trans_sequence_dict=fastaIter(transcriptFile)
	in_selectTrans=in_selectTrans.intersection(trans_sequence_dict.keys()).intersection(startCodonCoorDict.keys())
	with open(output_prefix+"_local_cds_sequences_RNA.fa",'w') as f1, open(output_prefix+"_local_cds_sequence_DNA.fa",'w') as f2, open(output_prefix+"_local_AA_sequences.fa",'w') as f3:
		for trans in in_selectTrans:
			trans_sequence=trans_sequence_dict[trans]
			startCoor=int(startCodonCoorDict[trans])-1 # 0-based
			stopCoor=int(stopCodonCoorDict[trans])-3 # 0-based , the first base of stop codon
			cds_sequence=trans_sequence[startCoor:(stopCoor+3)]
			AA_sequence=translation(cds_sequence,table=table,cds=False)
			local_cds_sequence=cds_sequence[3*(int(left_pos)-1):3*int(right_pos)]
			local_cds_sequence_RNA=local_cds_sequence.replace("T","U")
			local_AA_sequence=AA_sequence[(int(left_pos)-1):int(right_pos)]
			if len(local_AA_sequence) < (int(right_pos)-int(left_pos)+1):
				continue
			if len(local_cds_sequence_RNA) < (int(right_pos)-int(left_pos))*3+1:
				continue
			if len(local_cds_sequence) < (int(right_pos)-int(left_pos))*3+1:
				continue
			f1.write("%s%s\n%s\n" %(">",trans,local_cds_sequence_RNA))
			f2.write("%s%s\n%s\n" %(">",trans,local_cds_sequence))
			f3.write("%s%s\n%s\n" %(">",trans,local_AA_sequence))


def main():
	parser=create_parser_for_sequence_extraction()
	(options,args)=parser.parse_args()
	if not options.transcriptFile:
		raise IOError("Please enter the fasta file of all transcript sequences. Generated by RiboCode if in whole mode, or a given cds sequences in local mode.")
	if not options.coorFile:
		raise IOError("Please enter the longest.trans.info.txt file generated by OutputTranscriptInfo.py")
	selectTrans,transLengthDict,startCodonCoorDict,stopCodonCoorDict,transID2geneID,transID2geneName,cdsLengthDict,transID2ChromDict=reload_transcripts_information(options.coorFile)
	geneID2transID={v:k for k,v in transID2geneID.items()}
	geneName2transID={v:k for k,v in transID2geneName.items()}
	if options.in_selectTrans:
		select_trans=pd.read_csv(options.in_selectTrans,sep="\t")
		select_trans=set(select_trans.iloc[:,0].values)
		if options.id_type == 'transcript_id':
			select_trans=select_trans.intersection(selectTrans)
			print("There are " + str(len(select_trans)) + " transcripts from "+options.in_selectTrans+" used for following analysis.",file=sys.stderr)
		elif options.id_type == 'gene_id':
			tmp=[geneID2transID[gene_id] for gene_id in select_trans if gene_id in geneID2transID]
			select_trans=set(tmp)
			select_trans=select_trans.intersection(selectTrans)
			print("There are " + str(len(select_trans))+" gene id could be transformed into transcript id and used for following analysis.",file=sys.stderr)
		elif options.id_type == 'gene_name' or options.id_type=='gene_symbol':
			tmp=[geneName2transID[gene_name] for gene_name in select_trans if gene_name in geneName2transID]
			select_trans=set(tmp)
			select_trans=select_trans.intersection(selectTrans)
			print("There are " + str(len(select_trans))+" gene symbol could be transformed into transcript id and used for following analysis.",file=sys.stderr)
		else:
			raise IOError("Please input a approproate id_type parameters.[transcript_id/gene_id/gene_name/]")
	else:
		select_trans=selectTrans
	if options.mode == "whole":
		extract_protein_coding_sequence(options.transcriptFile,startCodonCoorDict,stopCodonCoorDict,select_trans,options.output_prefix,options.geneticCode)
		print("Finish the step of extracting sequences!",file=sys.stderr)
	elif options.mode == "local":
		extract_local_cds_sequence(options.transcriptFile,startCodonCoorDict,stopCodonCoorDict,select_trans,options.output_prefix,options.left_position,options.right_position,options.geneticCode)
		print("Finish the step of extracting sequences!",file=sys.stderr)
	else:
		raise IOError("Please enter a correct --mode parameter! [whole/local]")


if __name__=="__main__":
	main()




#!/usr/bin/env python
# -*- coding:UTF-8 -*-
'''
@Description: The script is used for plot the density around a specific di- or tri-AA motif.
'''

import sys
import numpy as np
import pandas as pd
from functools import reduce
from collections import defaultdict
import matplotlib
matplotlib.use('Agg')
import matplotlib.pyplot as plt
import seaborn as sns
from matplotlib.backends.backend_pdf import PdfPages
from optparse import OptionParser
from .__init__ import __version__


def create_parse_for_AAmotifs():
	usage="usage: python %prog [options]" + "\n"
	parser=OptionParser(usage=usage,version=__version__)
	parser.add_option('-i','--input',action='store',type='string',dest='density_file',
		help='Input file in dataframe format (pd.DataFrame).')
	parser.add_option('-o','--output',action='store',type='string',dest='output_prefix',
		help='Output files')
	parser.add_option('-g','--group',action="store",type="string",dest="group_name",
		help="Group name of each group separated by comma. e.g. 'si-control,si-eIF3e'")
	parser.add_option('-r','--replicate',action="store",type="string",dest="replicate_name",
		help="Replicate name of each group separated by comma. e.g. 'si_3e_1_80S,si_3e_2_80S__si_cttl_1_80S,si_ctrl_2_80S'")
	parser.add_option("--ymax",action="store",type="float",dest="ymax",default=None,
		help="The max of ylim. default=%default")
	parser.add_option("--ymin",action="store",type="float",dest="ymin",default=None,
		help="The min of ylim. default=%default")
	parser.add_option('--mode',action="store",type="string",dest="mode",default="mean",
		help="Control the mode for plot. if '--mode single', return the plot of each samples;else return the mean plot. default=%default.")
	return parser

def DrawMotifDensity_for_replicates_of_different_groups(data,motifs,groups,replicates,output,ymin,ymax,distance=101):
	''' plot the motif density of dfiierent replicates from different groups'''
	## prepare data for plot
	label_dict={}
	data_dict={}
	for g,r in zip(groups,replicates):
		label_dict[g]=r.strip().split(',')
	## separate the data into different groups
	for g in groups:
		columns=['motif']+label_dict[g]
		data_dict[g]=data.loc[:,columns]
	plt.rc('font',weight='bold')
	# colors=['blue','orangered']
	if len(groups) <=8:
		colors=["b","orangered","green","c","m","y","k","w"]
	else:
		colors=colors=sns.color_palette('husl',len(groups))
	text_font={"size":15,"family":"Arial","weight":"bold"}
	with PdfPages(output + "_density_on_triAAMotifs.pdf") as pdf:
		x=np.arange(distance,dtype="int64")
		for motif in motifs:
			fig=plt.figure(figsize=(5,4))
			ax=fig.add_subplot(111)
			for g in groups:
				for i in np.arange(len(label_dict[g])):
					plt.plot(x,data_dict[g][data_dict[g].iloc[:,0]==motif].loc[:,label_dict[g][i]],linewidth=1,color=colors[i],label=label_dict[g][i],alpha=0.8)
			ax.set_xlabel('Distance of P/E site to ' + str(motif)+' (nt)',fontdict=text_font)
			ax.set_ylabel('Relative footprint density(A.U)',fontdict=text_font)
			ax.spines['top'].set_visible(False)
			ax.spines['right'].set_visible(False)
			ax.spines['left'].set_linewidth(2)
			ax.spines['bottom'].set_linewidth(2)
			ax.tick_params(which="both",width=2)
			if not ymin and not ymax:
				pass
			elif not ymin and ymax:
				ax.set_ylim(0,ymax)
			elif ymin and not ymax:
				raise IOError("Please offer the ymax parameter as well!")
			elif ymin and ymax:
				ax.set_ylim(ymin,ymax)
			else:
				raise IOError("Please enter correct ymin and ymax parameters!")
			plt.xticks(np.arange(0,distance,25),(np.arange(0,distance,25)-int((distance-1)/2)))
			plt.legend(loc='best',prop=text_font)
			plt.tight_layout()
			pdf.savefig(fig)
			plt.close()



def DrawMotifDensity_for_mean_denisty(data_mean,motifs,output,ymin,ymax,distance=101):
	''' plot the motif density of dfiierent replicates from different groups'''
	plt.rc('font',weight='bold')
	groups_name=data_mean.columns[1:]
	if len(groups_name) <=8:
		colors=["b","orangered","green","c","m","y","k","w"]
	else:
		colors=colors=sns.color_palette('husl',len(groups_name))
	text_font={"size":15,"family":"Arial","weight":"bold"}
	with PdfPages(output + "_mean_density_on_triAAMotifs.pdf") as pdf:
		x=np.arange(distance,dtype="int64")
		for motif in motifs:
			fig=plt.figure(figsize=(5,4))
			ax=fig.add_subplot(111)
			for g in np.arange(len(groups_name)):
				plt.plot(x,data_mean[data_mean.iloc[:,0]==motif].loc[:,groups_name[g]],linewidth=1,color=colors[g],label=groups_name[g],alpha=0.8)
			ax.set_xlabel('Distance of P/E site to ' + str(motif)+' (nt)',fontdict=text_font)
			ax.set_ylabel('Relative footprint density(A.U)',fontdict=text_font)
			ax.spines['top'].set_visible(False)
			ax.spines['right'].set_visible(False)
			ax.spines['left'].set_linewidth(2)
			ax.spines['bottom'].set_linewidth(2)
			ax.tick_params(which="both",width=2)
			if not ymin and not ymax:
				pass
			elif not ymin and ymax:
				ax.set_ylim(0,ymax)
			elif ymin and not ymax:
				raise IOError("Please offer the ymax parameter as well!")
			elif ymin and ymax:
				ax.set_ylim(ymin,ymax)
			else:
				raise IOError("Please enter correct ymin and ymax parameters!")
			plt.xticks(np.arange(0,distance,25),(np.arange(0,distance,25)-int((distance-1)/2)))
			plt.legend(loc='best',prop=text_font)
			plt.tight_layout()
			pdf.savefig(fig)
			plt.close()


def calculate_mean_density_for_triplete_AA_motif(data,groups,replicates,output_prefix):
	'''calculate the mean values among different replicates. the input data have N+1 columns, N is the number of samples
	1) col 1: motifs
	2) col2-colN: different samples
	'''
	label_dict={}
	data_dict={}
	data_mean_dict=defaultdict(dict)
	motifs=data.iloc[:,0]
	for g,r in zip(groups,replicates):
		label_dict[g]=r.strip().split(',')
	## separate the data into different groups
	for g in groups:
		data_dict[g]=data.loc[:,label_dict[g]]

	for g in groups:
		for i in np.arange(data.shape[0]):
			density=np.mean(data_dict[g].loc[i,label_dict[g]])
			data_mean_dict[g][i]=density
	## transform the dict to a python dataframe
	for g in groups:
		data_mean_dict[g]=pd.DataFrame(data_mean_dict[g],index=[g]).T
	## concatenate different data frame
	data_mean=pd.concat([v for v in data_mean_dict.values()],axis=1)
	data_mean=pd.concat((motifs,data_mean),axis=1)
	## write the mean density file
	data_mean.to_csv(output_prefix+"_mean_motifDensity_dataframe.txt",sep="\t",index=0)
	return data_mean


def parse_args_for_plot_density_of_triplete_AA_motif():
	parser=create_parse_for_AAmotifs()
	(options,args)=parser.parse_args()
	(data,output,groups,replicates,ymin,ymax,mode)=(options.density_file,options.output_prefix,options.group_name.strip().split(','),
	options.replicate_name.strip().split('__'),options.ymin,options.ymax,options.mode)
	##  input data
	data=pd.read_csv(data,sep='\t')
	motifs=np.unique(data.iloc[:,0].values)
	if mode == 'single':
		print("Start the step of plotting...",file=sys.stderr)
		DrawMotifDensity_for_replicates_of_different_groups(data,motifs,groups,replicates,output,ymin,ymax,distance=101)
		print('finishing the plot step!',file=sys.stderr)
	elif mode == 'mean':
		## calculate the mean value of different replicates
		print("Start the step of mean value calculation...",file=sys.stderr)
		data_mean=calculate_mean_density_for_triplete_AA_motif(data,groups,replicates,output)
		print('finishing the mean value calculation!',file=sys.stderr)
		print("Start the step of plotting...",file=sys.stderr)
		DrawMotifDensity_for_mean_denisty(data_mean,motifs,output,ymin,ymax,distance=101)
		print('finishing the plot step!',file=sys.stderr)
	else:
		raise IOError("Please reset your --mode parameter.[single or mean]")

def main():
	parse_args_for_plot_density_of_triplete_AA_motif()


if __name__=="__main__":
	main()



#!/usr/bin/env python
# -*- coding:UTF-8 -*-
import os, sys
from setuptools import setup
from RiboMiner import __version__

if sys.version_info.major != 3:
    sys.exit("RiboMiner can only be used with Python 3. You are currently "
             "running Python %d." % sys.version_info.major)

with open("README.md", "r",encoding="utf-8") as fh:
    long_description = fh.read()

setup(
    name='RiboMiner',
    version = __version__,
    description = 'A python toolset for mining multi-dimensional features of the translatome with ribosome profiling data',
    long_description=long_description,
    long_description_content_type="text/markdown",
    keywords="Analysis for ribosome profiling data.",
    url="https://github.com/xryanglab/RiboMiner",
    author = 'Li Fajin',
    author_email = 'sherkinglee@gmail.com',
    license='GPLv3.0',
    packages=['RiboMiner','data'],
    install_requires=[
                      'matplotlib>=2.1.0',
                      'numpy>=1.16.4',
                      'pandas>=0.24.2',
                      'pysam>=0.15.2',
                      'scipy>=1.1.0',
                      'seaborn>=0.8.1',
                      'biopython>=1.70',
                      'scipy>=1.1.0',
                      'HTSeq',
                      'RiboCode>=1.2.10',
                      ],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Science/Research',
        'Topic :: Scientific/Engineering :: Bio-Informatics',
        'Environment :: Console',
        'Programming Language :: Python :: 3',
        'Operating System :: Unix',
        ],
    python_requires='>=3.6',
    entry_points={

        'console_scripts': [
            #name determined the name of cmd line direct call
            'cAI=RiboMiner.cAI:main',
            'EnrichmentAnalysis=RiboMiner.EnrichmentAnalysis:main',
            'EnrichmentAnalysisForSingleTrans=RiboMiner.EnrichmentAnalysisForSingleTrans:main',
            'enrichmentMeanDensity=RiboMiner.enrichmentMeanDensity:main',
            'ExtractSequenceCenteredOnAPosition=RiboMiner.ExtractSequenceCenteredOnAPosition:main',
            'GCContent=RiboMiner.GCContent:main',
            'GetProteinCodingSequence=RiboMiner.GetProteinCodingSequence:main',
            'GetUTRSequences=RiboMiner.GetUTRSequences:main',
            'hydropathyCharge=RiboMiner.hydropathyCharge:main',
            'LengthDistribution=RiboMiner.LengthDistribution:main',
            'MergeSampleDensitys=RiboMiner.MergeSampleDensitys:main',
            'MetageneAnalysis=RiboMiner.MetageneAnalysis:main',
            'MetageneAnalysisForTheWholeRegions=RiboMiner.MetageneAnalysisForTheWholeRegions:main',
            'OutputTranscriptInfo=RiboMiner.OutputTranscriptInfo:main',
            'PausingScore=RiboMiner.PausingScore:main',
            'Periodicity=RiboMiner.Periodicity:main',
            'PlotEnrichmentRatio=RiboMiner.PlotEnrichmentRatio:main',
            'PlotGCContent=RiboMiner.PlotGCContent:main',
            'PlotHydropathyCharge=RiboMiner.PlotHydropathyCharge:main',
            'PlotMetageneAnalysis=RiboMiner.PlotMetageneAnalysis:main',
            'PlotMetageneAnalysisForTheWholeRegions=RiboMiner.PlotMetageneAnalysisForTheWholeRegions:main',
            'PlotPolarity=RiboMiner.PlotPolarity:main',
            'PlotRiboDensityAroundTriAAMotifs=RiboMiner.PlotRiboDensityAroundTriAAMotifs:main',
            'PlotRiboDensityAtEachKindAAOrCodon=RiboMiner.PlotRiboDensityAtEachKindAAOrCodon:main',
            'PolarityCalculation=RiboMiner.PolarityCalculation:main',
            'ProcessPausingScore=RiboMiner.ProcessPausingScore:main',
            'RiboDensityAroundTripleteAAMotifs=RiboMiner.RiboDensityAroundTripleteAAMotifs:main',
            'RiboDensityAtEachKindAAOrCodon=RiboMiner.RiboDensityAtEachKindAAOrCodon:main',
            'RiboDensityAtEachPosition=RiboMiner.RiboDensityAtEachPosition:main',
            'RiboDensityForSpecificRegion=RiboMiner.RiboDensityForSpecificRegion:main',
            'RiboDensityOfDiffFrames=RiboMiner.RiboDensityOfDiffFrames:main',
            'RPFdist=RiboMiner.RPFdist:main',
            'StatisticReadsOnDNAsContam=RiboMiner.StatisticReadsOnDNAsContam:main',
            'tAI=RiboMiner.tAI:main',
            'tAIPlot=RiboMiner.tAIPlot:main',
            'cAIPlot=RiboMiner.cAIPlot:main',
            'ModifyHTseq=RiboMiner.ModifyHTseq:main',
            'ReadsLengthOfSpecificRegions=RiboMiner.ReadsLengthOfSpecificRegions:main',
            'CoverageOfEachTrans=RiboMiner.CoverageOfEachTrans:main',
            'PlotTransCoverage=RiboMiner.PlotTransCoverage:main',
        ],

    },
        )